@@ -1,3 +1,5 @@
+#ifndef RUBY_ADDR2LINE_H
+#define RUBY_ADDR2LINE_H
 /**********************************************************************
 
   addr2line.h -
@@ -8,13 +10,12 @@
 
 **********************************************************************/
 
-#ifndef RUBY_ADDR2LINE_H
-#define RUBY_ADDR2LINE_H
+#if (defined(USE_ELF) || defined(HAVE_MACH_O_LOADER_H))
 
-#ifdef USE_ELF
+#include <stdio.h>
 
 void
-rb_dump_backtrace_with_lines(int num_traces, void **traces, char **syms);
+rb_dump_backtrace_with_lines(int num_traces, void **traces, FILE *errout);
 
 #endif /* USE_ELF */
 
