@@ -589,6 +589,8 @@ assert_equal 'string', %q{
 
 # Check that exceptions work when getting global variable
 assert_equal 'rescued', %q{
+  Warning[:deprecated] = true
+
   module Warning
     def warn(message)
       raise
@@ -4508,3 +4510,10 @@ assert_equal 'ok', <<~'RUBY'
 
   :ok
 RUBY
+
+assert_normal_exit %{
+  class Bug20997
+    def foo(&) = self.class.name(&)
+    new.foo
+  end
+}
