@@ -8,72 +8,164 @@
 
 ************************************************/
 
-#include "gc.h"
+#include "internal/gc.h"
 #include "internal/hash.h"
 #include "internal/thread.h"
 #include "internal/sanitizers.h"
 #include "ruby.h"
 #include "vm_core.h"
 
+static enum {
+    IDLE,
+    SUSPENDED,
+    RUNNING
+} current_state = IDLE;
 static int current_mode;
 static VALUE me2counter = Qnil;
 
 /*
- * call-seq:
- *    Coverage.start  => nil
+ *  call-seq: Coverage.supported?(mode) -> true or false
  *
- * Enables coverage measurement.
+ *  Returns true if coverage measurement is supported for the given mode.
+ *
+ *  The mode should be one of the following symbols:
+ *  +:lines+, +:oneshot_lines+, +:branches+, +:methods+, +:eval+.
+ *
+ *  Example:
+ *
+ *    Coverage.supported?(:lines)  #=> true
+ *    Coverage.supported?(:all)    #=> false
  */
 static VALUE
-rb_coverage_start(int argc, VALUE *argv, VALUE klass)
+rb_coverage_supported(VALUE self, VALUE _mode)
+{
+    ID mode = RB_SYM2ID(_mode);
+
+    return RBOOL(
+        mode == rb_intern("lines") ||
+        mode == rb_intern("oneshot_lines") ||
+        mode == rb_intern("branches") ||
+        mode == rb_intern("methods") ||
+        mode == rb_intern("eval")
+    );
+}
+
+/*
+ * call-seq:
+ *    Coverage.setup                                                          => nil
+ *    Coverage.setup(:all)                                                    => nil
+ *    Coverage.setup(lines: bool, branches: bool, methods: bool, eval: bool)  => nil
+ *    Coverage.setup(oneshot_lines: true)                                     => nil
+ *
+ * Set up the coverage measurement.
+ *
+ * Note that this method does not start the measurement itself.
+ * Use Coverage.resume to start the measurement.
+ *
+ * You may want to use Coverage.start to setup and then start the measurement.
+ */
+static VALUE
+rb_coverage_setup(int argc, VALUE *argv, VALUE klass)
 {
     VALUE coverages, opt;
     int mode;
 
+    if (current_state != IDLE) {
+        rb_raise(rb_eRuntimeError, "coverage measurement is already setup");
+    }
+
     rb_scan_args(argc, argv, "01", &opt);
 
     if (argc == 0) {
-	mode = 0; /* compatible mode */
+        mode = 0; /* compatible mode */
     }
     else if (opt == ID2SYM(rb_intern("all"))) {
-	mode = COVERAGE_TARGET_LINES | COVERAGE_TARGET_BRANCHES | COVERAGE_TARGET_METHODS;
+        mode = COVERAGE_TARGET_LINES | COVERAGE_TARGET_BRANCHES | COVERAGE_TARGET_METHODS | COVERAGE_TARGET_EVAL;
     }
     else {
-	mode = 0;
-	opt = rb_convert_type(opt, T_HASH, "Hash", "to_hash");
+        mode = 0;
+        opt = rb_convert_type(opt, T_HASH, "Hash", "to_hash");
 
-	if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("lines")))))
-	    mode |= COVERAGE_TARGET_LINES;
-	if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("branches")))))
-	    mode |= COVERAGE_TARGET_BRANCHES;
-	if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("methods")))))
-	    mode |= COVERAGE_TARGET_METHODS;
+        if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("lines")))))
+            mode |= COVERAGE_TARGET_LINES;
+        if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("branches")))))
+            mode |= COVERAGE_TARGET_BRANCHES;
+        if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("methods")))))
+            mode |= COVERAGE_TARGET_METHODS;
         if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("oneshot_lines"))))) {
             if (mode & COVERAGE_TARGET_LINES)
                 rb_raise(rb_eRuntimeError, "cannot enable lines and oneshot_lines simultaneously");
             mode |= COVERAGE_TARGET_LINES;
             mode |= COVERAGE_TARGET_ONESHOT_LINES;
         }
+        if (RTEST(rb_hash_lookup(opt, ID2SYM(rb_intern("eval")))))
+            mode |= COVERAGE_TARGET_EVAL;
     }
 
     if (mode & COVERAGE_TARGET_METHODS) {
         me2counter = rb_ident_hash_new();
     }
     else {
-	me2counter = Qnil;
+        me2counter = Qnil;
     }
 
     coverages = rb_get_coverages();
     if (!RTEST(coverages)) {
-	coverages = rb_hash_new();
-	rb_obj_hide(coverages);
-	current_mode = mode;
-	if (mode == 0) mode = COVERAGE_TARGET_LINES;
-	rb_set_coverages(coverages, mode, me2counter);
+        coverages = rb_hash_new();
+        rb_obj_hide(coverages);
+        current_mode = mode;
+        if (mode == 0) mode = COVERAGE_TARGET_LINES;
+        rb_set_coverages(coverages, mode, me2counter);
+        current_state = SUSPENDED;
     }
     else if (current_mode != mode) {
-	rb_raise(rb_eRuntimeError, "cannot change the measuring target during coverage measurement");
+        rb_raise(rb_eRuntimeError, "cannot change the measuring target during coverage measurement");
     }
+
+    return Qnil;
+}
+
+/*
+ * call-seq:
+ *    Coverage.resume  => nil
+ *
+ * Start/resume the coverage measurement.
+ *
+ * Caveat: Currently, only process-global coverage measurement is supported.
+ * You cannot measure per-thread coverage. If your process has multiple thread,
+ * using Coverage.resume/suspend to capture code coverage executed from only
+ * a limited code block, may yield misleading results.
+ */
+VALUE
+rb_coverage_resume(VALUE klass)
+{
+    if (current_state == IDLE) {
+        rb_raise(rb_eRuntimeError, "coverage measurement is not set up yet");
+    }
+    if (current_state == RUNNING) {
+        rb_raise(rb_eRuntimeError, "coverage measurement is already running");
+    }
+    rb_resume_coverages();
+    current_state = RUNNING;
+    return Qnil;
+}
+
+/*
+ * call-seq:
+ *    Coverage.start                                                          => nil
+ *    Coverage.start(:all)                                                    => nil
+ *    Coverage.start(lines: bool, branches: bool, methods: bool, eval: bool)  => nil
+ *    Coverage.start(oneshot_lines: true)                                     => nil
+ *
+ * Enables the coverage measurement.
+ * See the documentation of Coverage class in detail.
+ * This is equivalent to Coverage.setup and Coverage.resume.
+ */
+static VALUE
+rb_coverage_start(int argc, VALUE *argv, VALUE klass)
+{
+    rb_coverage_setup(argc, argv, klass);
+    rb_coverage_resume(klass);
     return Qnil;
 }
 
@@ -154,45 +246,45 @@ method_coverage_i(void *vstart, void *vend, size_t stride, void *data)
         void *poisoned = asan_poisoned_object_p(v);
         asan_unpoison_object(v, false);
 
-	if (RB_TYPE_P(v, T_IMEMO) && imemo_type(v) == imemo_ment) {
-	    const rb_method_entry_t *me = (rb_method_entry_t *) v;
-	    VALUE path, first_lineno, first_column, last_lineno, last_column;
-	    VALUE data[5], ncoverage, methods;
-	    VALUE methods_id = ID2SYM(rb_intern("methods"));
-	    VALUE klass;
-	    const rb_method_entry_t *me2 = rb_resolve_me_location(me, data);
-	    if (me != me2) continue;
-	    klass = me->owner;
-	    if (RB_TYPE_P(klass, T_ICLASS)) {
-		rb_bug("T_ICLASS");
-	    }
-	    path = data[0];
-	    first_lineno = data[1];
-	    first_column = data[2];
-	    last_lineno = data[3];
-	    last_column = data[4];
-	    if (FIX2LONG(first_lineno) <= 0) continue;
-	    ncoverage = rb_hash_aref(ncoverages, path);
-	    if (NIL_P(ncoverage)) continue;
-	    methods = rb_hash_aref(ncoverage, methods_id);
+        if (RB_TYPE_P(v, T_IMEMO) && imemo_type(v) == imemo_ment) {
+            const rb_method_entry_t *me = (rb_method_entry_t *) v;
+            VALUE path, first_lineno, first_column, last_lineno, last_column;
+            VALUE data[5], ncoverage, methods;
+            VALUE methods_id = ID2SYM(rb_intern("methods"));
+            VALUE klass;
+            const rb_method_entry_t *me2 = rb_resolve_me_location(me, data);
+            if (me != me2) continue;
+            klass = me->owner;
+            if (RB_TYPE_P(klass, T_ICLASS)) {
+                rb_bug("T_ICLASS");
+            }
+            path = data[0];
+            first_lineno = data[1];
+            first_column = data[2];
+            last_lineno = data[3];
+            last_column = data[4];
+            if (FIX2LONG(first_lineno) <= 0) continue;
+            ncoverage = rb_hash_aref(ncoverages, path);
+            if (NIL_P(ncoverage)) continue;
+            methods = rb_hash_aref(ncoverage, methods_id);
 
-	    {
-		VALUE method_id = ID2SYM(me->def->original_id);
-		VALUE rcount = rb_hash_aref(me2counter, (VALUE) me);
-		VALUE key = rb_ary_new_from_args(6, klass, method_id, first_lineno, first_column, last_lineno, last_column);
-		VALUE rcount2 = rb_hash_aref(methods, key);
+            {
+                VALUE method_id = ID2SYM(me->def->original_id);
+                VALUE rcount = rb_hash_aref(me2counter, (VALUE) me);
+                VALUE key = rb_ary_new_from_args(6, klass, method_id, first_lineno, first_column, last_lineno, last_column);
+                VALUE rcount2 = rb_hash_aref(methods, key);
 
-		if (NIL_P(rcount)) rcount = LONG2FIX(0);
-		if (NIL_P(rcount2)) rcount2 = LONG2FIX(0);
-		if (!POSFIXABLE(FIX2LONG(rcount) + FIX2LONG(rcount2))) {
-		    rcount = LONG2FIX(FIXNUM_MAX);
-		}
-		else {
-		    rcount = LONG2FIX(FIX2LONG(rcount) + FIX2LONG(rcount2));
-		}
-		rb_hash_aset(methods, key, rcount);
-	    }
-	}
+                if (NIL_P(rcount)) rcount = LONG2FIX(0);
+                if (NIL_P(rcount2)) rcount2 = LONG2FIX(0);
+                if (!POSFIXABLE(FIX2LONG(rcount) + FIX2LONG(rcount2))) {
+                    rcount = LONG2FIX(FIXNUM_MAX);
+                }
+                else {
+                    rcount = LONG2FIX(FIX2LONG(rcount) + FIX2LONG(rcount2));
+                }
+                rb_hash_aset(methods, key, rcount);
+            }
+        }
 
         if (poisoned) {
             asan_poison_object(v);
@@ -208,32 +300,32 @@ coverage_peek_result_i(st_data_t key, st_data_t val, st_data_t h)
     VALUE coverage = (VALUE)val;
     VALUE coverages = (VALUE)h;
     if (current_mode == 0) {
-	/* compatible mode */
-	VALUE lines = rb_ary_dup(RARRAY_AREF(coverage, COVERAGE_INDEX_LINES));
-	rb_ary_freeze(lines);
-	coverage = lines;
+        /* compatible mode */
+        VALUE lines = rb_ary_dup(RARRAY_AREF(coverage, COVERAGE_INDEX_LINES));
+        rb_ary_freeze(lines);
+        coverage = lines;
     }
     else {
-	VALUE h = rb_hash_new();
+        VALUE h = rb_hash_new();
 
-	if (current_mode & COVERAGE_TARGET_LINES) {
-	    VALUE lines = RARRAY_AREF(coverage, COVERAGE_INDEX_LINES);
+        if (current_mode & COVERAGE_TARGET_LINES) {
+            VALUE lines = RARRAY_AREF(coverage, COVERAGE_INDEX_LINES);
             const char *kw = (current_mode & COVERAGE_TARGET_ONESHOT_LINES) ? "oneshot_lines" : "lines";
-	    lines = rb_ary_dup(lines);
-	    rb_ary_freeze(lines);
+            lines = rb_ary_dup(lines);
+            rb_ary_freeze(lines);
             rb_hash_aset(h, ID2SYM(rb_intern(kw)), lines);
-	}
+        }
 
-	if (current_mode & COVERAGE_TARGET_BRANCHES) {
-	    VALUE branches = RARRAY_AREF(coverage, COVERAGE_INDEX_BRANCHES);
-	    rb_hash_aset(h, ID2SYM(rb_intern("branches")), branch_coverage(branches));
-	}
+        if (current_mode & COVERAGE_TARGET_BRANCHES) {
+            VALUE branches = RARRAY_AREF(coverage, COVERAGE_INDEX_BRANCHES);
+            rb_hash_aset(h, ID2SYM(rb_intern("branches")), branch_coverage(branches));
+        }
 
-	if (current_mode & COVERAGE_TARGET_METHODS) {
-	    rb_hash_aset(h, ID2SYM(rb_intern("methods")), rb_hash_new());
-	}
+        if (current_mode & COVERAGE_TARGET_METHODS) {
+            rb_hash_aset(h, ID2SYM(rb_intern("methods")), rb_hash_new());
+        }
 
-	coverage = h;
+        coverage = h;
     }
 
     rb_hash_aset(coverages, path, coverage);
@@ -258,13 +350,13 @@ rb_coverage_peek_result(VALUE klass)
     VALUE coverages = rb_get_coverages();
     VALUE ncoverages = rb_hash_new();
     if (!RTEST(coverages)) {
-	rb_raise(rb_eRuntimeError, "coverage measurement is not enabled");
+        rb_raise(rb_eRuntimeError, "coverage measurement is not enabled");
     }
     OBJ_WB_UNPROTECT(coverages);
     st_foreach(RHASH_TBL_RAW(coverages), coverage_peek_result_i, ncoverages);
 
     if (current_mode & COVERAGE_TARGET_METHODS) {
-	rb_objspace_each_objects(method_coverage_i, &ncoverages);
+        rb_objspace_each_objects(method_coverage_i, &ncoverages);
     }
 
     rb_hash_freeze(ncoverages);
@@ -279,6 +371,24 @@ clear_me2counter_i(VALUE key, VALUE value, VALUE unused)
     return ST_CONTINUE;
 }
 
+/*
+ * call-seq:
+ *    Coverage.suspend  => nil
+ *
+ * Suspend the coverage measurement.
+ * You can use Coverage.resume to restart the measurement.
+ */
+VALUE
+rb_coverage_suspend(VALUE klass)
+{
+    if (current_state != RUNNING) {
+        rb_raise(rb_eRuntimeError, "coverage measurement is not running");
+    }
+    rb_suspend_coverages();
+    current_state = SUSPENDED;
+    return Qnil;
+}
+
 /*
  *  call-seq:
  *     Coverage.result(stop: true, clear: true)  => hash
@@ -294,6 +404,10 @@ rb_coverage_result(int argc, VALUE *argv, VALUE klass)
     VALUE opt;
     int stop = 1, clear = 1;
 
+    if (current_state == IDLE) {
+        rb_raise(rb_eRuntimeError, "coverage measurement is not enabled");
+    }
+
     rb_scan_args(argc, argv, "01", &opt);
 
     if (argc == 1) {
@@ -312,13 +426,34 @@ rb_coverage_result(int argc, VALUE *argv, VALUE klass)
         if (!NIL_P(me2counter)) rb_hash_foreach(me2counter, clear_me2counter_i, Qnil);
     }
     if (stop) {
+        if (current_state == RUNNING) {
+            rb_coverage_suspend(klass);
+        }
         rb_reset_coverages();
         me2counter = Qnil;
+        current_state = IDLE;
     }
     return ncoverages;
 }
 
 
+/*
+ *  call-seq:
+ *     Coverage.state  => :idle, :suspended, :running
+ *
+ * Returns the state of the coverage measurement.
+ */
+static VALUE
+rb_coverage_state(VALUE klass)
+{
+    switch (current_state) {
+        case IDLE: return ID2SYM(rb_intern("idle"));
+        case SUSPENDED: return ID2SYM(rb_intern("suspended"));
+        case RUNNING: return ID2SYM(rb_intern("running"));
+    }
+    return Qnil;
+}
+
 /*
  *  call-seq:
  *     Coverage.running?  => bool
@@ -329,13 +464,15 @@ rb_coverage_result(int argc, VALUE *argv, VALUE klass)
 static VALUE
 rb_coverage_running(VALUE klass)
 {
-    VALUE coverages = rb_get_coverages();
-    return RTEST(coverages) ? Qtrue : Qfalse;
+    return current_state == RUNNING ? Qtrue : Qfalse;
 }
 
 /* Coverage provides coverage measurement feature for Ruby.
  * This feature is experimental, so these APIs may be changed in future.
  *
+ * Caveat: Currently, only process-global coverage measurement is supported.
+ * You cannot measure per-thread coverage.
+ *
  * = Usage
  *
  * 1. require "coverage"
@@ -480,9 +617,16 @@ void
 Init_coverage(void)
 {
     VALUE rb_mCoverage = rb_define_module("Coverage");
+
+    rb_define_singleton_method(rb_mCoverage, "supported?", rb_coverage_supported, 1);
+
+    rb_define_module_function(rb_mCoverage, "setup", rb_coverage_setup, -1);
     rb_define_module_function(rb_mCoverage, "start", rb_coverage_start, -1);
+    rb_define_module_function(rb_mCoverage, "resume", rb_coverage_resume, 0);
+    rb_define_module_function(rb_mCoverage, "suspend", rb_coverage_suspend, 0);
     rb_define_module_function(rb_mCoverage, "result", rb_coverage_result, -1);
     rb_define_module_function(rb_mCoverage, "peek_result", rb_coverage_peek_result, 0);
+    rb_define_module_function(rb_mCoverage, "state", rb_coverage_state, 0);
     rb_define_module_function(rb_mCoverage, "running?", rb_coverage_running, 0);
     rb_global_variable(&me2counter);
 }
