@@ -1,6 +1,17 @@
 # frozen_string_literal: false
 require 'mkmf'
 
+version = ["../../..", "."].find do |dir|
+  break File.read(File.join(__dir__, dir, "io-console.gemspec"))[/^_VERSION\s*=\s*"(.*?)"/, 1]
+rescue
+end
+
+have_func("rb_io_path")
+have_func("rb_io_descriptor")
+have_func("rb_io_get_write_io")
+have_func("rb_io_closed_p")
+have_func("rb_io_open_descriptor")
+
 ok = true if RUBY_ENGINE == "ruby" || RUBY_ENGINE == "truffleruby"
 hdr = nil
 case
@@ -29,7 +40,7 @@ when true
   elsif have_func("rb_scheduler_timeout") # 3.0
     have_func("rb_io_wait")
   end
-  $defs << "-D""ENABLE_IO_GETPASS=1"
+  $defs << "-D""IO_CONSOLE_VERSION=#{version}"
   create_makefile("io/console") {|conf|
     conf << "\n""VK_HEADER = #{vk_header}\n"
   }
