@@ -22,11 +22,6 @@ rb_call_inits(void)
 {
     CALL(default_shapes);
     CALL(Thread_Mutex);
-#if USE_TRANSIENT_HEAP
-    CALL(TransientHeap);
-#endif
-    CALL(vm_postponed_job);
-    CALL(Method);
     CALL(RandomSeedCore);
     CALL(encodings);
     CALL(sym);
@@ -56,17 +51,18 @@ rb_call_inits(void)
     CALL(Dir);
     CALL(Time);
     CALL(Random);
-    CALL(signal);
     CALL(load);
     CALL(Proc);
     CALL(Binding);
     CALL(Math);
     CALL(GC);
+    CALL(WeakMap);
     CALL(Enumerator);
     CALL(Ractor);
     CALL(VM);
     CALL(ISeq);
     CALL(Thread);
+    CALL(signal);
     CALL(Fiber_Scheduler);
     CALL(process);
     CALL(Cont);
@@ -79,6 +75,7 @@ rb_call_inits(void)
     CALL(ast);
     CALL(gc_stress);
     CALL(shape);
+    CALL(Prism);
 
     // enable builtin loading
     CALL(builtin);
@@ -105,9 +102,9 @@ rb_call_builtin_inits(void)
     BUILTIN(yjit);
     BUILTIN(nilclass);
     BUILTIN(marshal);
-#if USE_MJIT
-    BUILTIN(mjit);
-    BUILTIN(mjit_c);
+#if USE_RJIT
+    BUILTIN(rjit_c);
+    BUILTIN(rjit);
 #endif
     Init_builtin_prelude();
 }
