@@ -7,9 +7,7 @@ require 'rbconfig'
 require 'fileutils'
 require 'shellwords'
 
-class String
-  # :stopdoc:
-
+class String # :nodoc:
   # Wraps a string in escaped quotes if it contains whitespace.
   def quote
     /\s/ =~ self ? "\"#{self}\"" : "#{self}"
@@ -32,19 +30,13 @@ class String
   def sans_arguments
     self[/\A[^()]+/]
   end
-
-  # :startdoc:
 end
 
-class Array
-  # :stopdoc:
-
+class Array # :nodoc:
   # Wraps all strings in escaped quotes if they contain whitespace.
   def quote
     map {|s| s.quote}
   end
-
-  # :startdoc:
 end
 
 ##
@@ -75,7 +67,7 @@ module MakeMakefile
   C_EXT = %w[c m]
 
   ##
-  # Extensions for files complied with a C++ compiler
+  # Extensions for files compiled with a C++ compiler
 
   CXX_EXT = %w[cc mm cxx cpp]
   unless File.exist?(File.join(*File.split(__FILE__).tap {|d, b| b.swapcase}))
@@ -97,26 +89,16 @@ module MakeMakefile
 
   unless defined? $configure_args
     $configure_args = {}
-    args = CONFIG["configure_args"]
-    if ENV["CONFIGURE_ARGS"]
-      args << " " << ENV["CONFIGURE_ARGS"]
+    args = CONFIG["configure_args"].shellsplit
+    if arg = ENV["CONFIGURE_ARGS"]
+      args.push(*arg.shellsplit)
     end
-    for arg in Shellwords::shellwords(args)
+    args.delete_if {|a| /\A--(?:top(?:src)?|src|cur)dir(?=\z|=)/ =~ a}
+    for arg in args.concat(ARGV)
       arg, val = arg.split('=', 2)
       next unless arg
       arg.tr!('_', '-')
-      if arg.sub!(/^(?!--)/, '--')
-        val or next
-        arg.downcase!
-      end
-      next if /^--(?:top|topsrc|src|cur)dir$/ =~ arg
-      $configure_args[arg] = val || true
-    end
-    for arg in ARGV
-      arg, val = arg.split('=', 2)
-      next unless arg
-      arg.tr!('_', '-')
-      if arg.sub!(/^(?!--)/, '--')
+      if arg.sub!(/\A(?!--)/, '--')
         val or next
         arg.downcase!
       end
@@ -688,16 +670,6 @@ MSG
     try_compile(MAIN_DOES_NOTHING, flags, {:werror => true}.update(opts))
   end
 
-  def append_cflags(flags, *opts)
-    Array(flags).each do |flag|
-      if checking_for("whether #{flag} is accepted as CFLAGS") {
-           try_cflags(flag, *opts)
-         }
-        $CFLAGS << " " << flag
-      end
-    end
-  end
-
   def with_ldflags(flags)
     ldflags = $LDFLAGS
     $LDFLAGS = flags.dup
@@ -999,7 +971,11 @@ SRC
   # Internal use only.
   #
   def checking_for(m, fmt = nil)
-    f = caller[0][/in `([^<].*)'$/, 1] and f << ": " #` for vim #'
+    if f = caller_locations(1, 1).first.base_label and /\A\w/ =~ f
+      f += ": "
+    else
+      f = ""
+    end
     m = "checking #{/\Acheck/ =~ f ? '' : 'for '}#{m}... "
     message "%s", m
     a = r = nil
@@ -1032,6 +1008,21 @@ SRC
 
   # :startdoc:
 
+  # Check whether each given C compiler flag is acceptable and append it
+  # to <tt>$CFLAGS</tt> if so.
+  #
+  # [+flags+] a C compiler flag as a +String+ or an +Array+ of them
+  #
+  def append_cflags(flags, *opts)
+    Array(flags).each do |flag|
+      if checking_for("whether #{flag} is accepted as CFLAGS") {
+           try_cflags(flag, *opts)
+         }
+        $CFLAGS << " " << flag
+      end
+    end
+  end
+
   # Returns whether or not +macro+ is defined either in the common header
   # files or within any +headers+ you provide.
   #
@@ -1359,8 +1350,10 @@ SRC
 
   # :stopdoc:
   STRING_OR_FAILED_FORMAT = "%s"
-  def STRING_OR_FAILED_FORMAT.%(x) # :nodoc:
-    x ? super : "failed"
+  class << STRING_OR_FAILED_FORMAT # :nodoc:
+    def %(x)
+      x ? super : "failed"
+    end
   end
 
   def typedef_expr(type, headers)
@@ -1763,7 +1756,7 @@ SRC
     hdr << "#endif\n"
     hdr = hdr.join("")
     log_src(hdr, "#{header} is")
-    unless (IO.read(header) == hdr rescue false)
+    unless (File.read(header) == hdr rescue false)
       File.open(header, "wb") do |hfile|
         hfile.write(hdr)
       end
@@ -1799,7 +1792,8 @@ SRC
   # application.
   #
   def dir_config(target, idefault=nil, ldefault=nil)
-    if conf = $config_dirs[target]
+    key = [target, idefault, ldefault].compact.join("\0")
+    if conf = $config_dirs[key]
       return conf
     end
 
@@ -1809,9 +1803,13 @@ SRC
     end
 
     idir = with_config(target + "-include", idefault)
-    $arg_config.last[1] ||= "${#{target}-dir}/include"
+    if conf = $arg_config.assoc("--with-#{target}-include")
+      conf[1] ||= "${#{target}-dir}/include"
+    end
     ldir = with_config(target + "-lib", ldefault)
-    $arg_config.last[1] ||= "${#{target}-dir}/#{_libdir_basename}"
+    if conf = $arg_config.assoc("--with-#{target}-lib")
+      conf[1] ||= "${#{target}-dir}/#{_libdir_basename}"
+    end
 
     idirs = idir ? Array === idir ? idir.dup : idir.split(File::PATH_SEPARATOR) : []
     if defaults
@@ -1833,92 +1831,96 @@ SRC
     end
     $LIBPATH = ldirs | $LIBPATH
 
-    $config_dirs[target] = [idir, ldir]
+    $config_dirs[key] = [idir, ldir]
   end
 
-  # Returns compile/link information about an installed library in a
-  # tuple of <code>[cflags, ldflags, libs]</code>, by using the
-  # command found first in the following commands:
+  # Returns compile/link information about an installed library in a tuple of <code>[cflags,
+  # ldflags, libs]</code>, by using the command found first in the following commands:
   #
   # 1. If <code>--with-{pkg}-config={command}</code> is given via
-  #    command line option: <code>{command} {option}</code>
+  #    command line option: <code>{command} {options}</code>
   #
-  # 2. <code>{pkg}-config {option}</code>
+  # 2. <code>{pkg}-config {options}</code>
   #
-  # 3. <code>pkg-config {option} {pkg}</code>
+  # 3. <code>pkg-config {options} {pkg}</code>
   #
-  # Where {option} is, for instance, <code>--cflags</code>.
+  # Where +options+ is the option name without dashes, for instance <code>"cflags"</code> for the
+  # <code>--cflags</code> flag.
   #
-  # The values obtained are appended to +$INCFLAGS+, +$CFLAGS+, +$LDFLAGS+ and
-  # +$libs+.
+  # The values obtained are appended to <code>$INCFLAGS</code>, <code>$CFLAGS</code>,
+  # <code>$LDFLAGS</code> and <code>$libs</code>.
   #
-  # If an <code>option</code> argument is given, the config command is
-  # invoked with the option and a stripped output string is returned
-  # without modifying any of the global values mentioned above.
-  def pkg_config(pkg, option=nil)
-    _, ldir = dir_config(pkg)
-    if ldir
-      pkg_config_path = "#{ldir}/pkgconfig"
-      if File.directory?(pkg_config_path)
-        Logging.message("PKG_CONFIG_PATH = %s\n", pkg_config_path)
-        envs = ["PKG_CONFIG_PATH"=>[pkg_config_path, ENV["PKG_CONFIG_PATH"]].compact.join(File::PATH_SEPARATOR)]
-      end
+  # If one or more <code>options</code> argument is given, the config command is
+  # invoked with the options and a stripped output string is returned without
+  # modifying any of the global values mentioned above.
+  def pkg_config(pkg, *options)
+    fmt = "not found"
+    def fmt.%(x)
+      x ? x.inspect : self
     end
-    if pkgconfig = with_config("#{pkg}-config") and find_executable0(pkgconfig)
-      # if and only if package specific config command is given
-    elsif ($PKGCONFIG ||=
-           (pkgconfig = with_config("pkg-config", ("pkg-config" unless CROSS_COMPILING))) &&
-           find_executable0(pkgconfig) && pkgconfig) and
-        xsystem([*envs, $PKGCONFIG, "--exists", pkg])
-      # default to pkg-config command
-      pkgconfig = $PKGCONFIG
-      get = proc {|opt|
-        opt = xpopen([*envs, $PKGCONFIG, "--#{opt}", pkg], err:[:child, :out], &:read)
-        Logging.open {puts opt.each_line.map{|s|"=> #{s.inspect}"}}
-        opt.strip if $?.success?
-      }
-    elsif find_executable0(pkgconfig = "#{pkg}-config")
-      # default to package specific config command, as a last resort.
-    else
-      pkgconfig = nil
-    end
-    if pkgconfig
-      get ||= proc {|opt|
-        opt = xpopen([*envs, pkgconfig, "--#{opt}"], err:[:child, :out], &:read)
-        Logging.open {puts opt.each_line.map{|s|"=> #{s.inspect}"}}
-        opt.strip if $?.success?
-      }
-    end
-    orig_ldflags = $LDFLAGS
-    if get and option
-      get[option]
-    elsif get and try_ldflags(ldflags = get['libs'])
-      if incflags = get['cflags-only-I']
-        $INCFLAGS << " " << incflags
-        cflags = get['cflags-only-other']
-      else
-        cflags = get['cflags']
-      end
-      libs = get['libs-only-l']
-      if cflags
-        $CFLAGS += " " << cflags
-        $CXXFLAGS += " " << cflags
-      end
-      if libs
-        ldflags = (Shellwords.shellwords(ldflags) - Shellwords.shellwords(libs)).quote.join(" ")
-      else
-        libs, ldflags = Shellwords.shellwords(ldflags).partition {|s| s =~ /-l([^ ]+)/ }.map {|l|l.quote.join(" ")}
-      end
-      $libs += " " << libs
 
-      $LDFLAGS = [orig_ldflags, ldflags].join(' ')
-      Logging::message "package configuration for %s\n", pkg
-      Logging::message "incflags: %s\ncflags: %s\nldflags: %s\nlibs: %s\n\n",
-                       incflags, cflags, ldflags, libs
-      [[incflags, cflags].join(' '), ldflags, libs]
-    else
-      Logging::message "package configuration for %s is not found\n", pkg
-      nil
+    checking_for "pkg-config for #{pkg}", fmt do
+      _, ldir = dir_config(pkg)
+      if ldir
+        pkg_config_path = "#{ldir}/pkgconfig"
+        if File.directory?(pkg_config_path)
+          Logging.message("PKG_CONFIG_PATH = %s\n", pkg_config_path)
+          envs = ["PKG_CONFIG_PATH"=>[pkg_config_path, ENV["PKG_CONFIG_PATH"]].compact.join(File::PATH_SEPARATOR)]
+        end
+      end
+      if pkgconfig = with_config("#{pkg}-config") and find_executable0(pkgconfig)
+      # if and only if package specific config command is given
+      elsif ($PKGCONFIG ||=
+             (pkgconfig = with_config("pkg-config") {config_string("PKG_CONFIG") || "pkg-config"}) &&
+             find_executable0(pkgconfig) && pkgconfig) and
+           xsystem([*envs, $PKGCONFIG, "--exists", pkg])
+        # default to pkg-config command
+        pkgconfig = $PKGCONFIG
+        args = [pkg]
+      elsif find_executable0(pkgconfig = "#{pkg}-config")
+      # default to package specific config command, as a last resort.
+      else
+        pkgconfig = nil
+      end
+      if pkgconfig
+        get = proc {|opts|
+          opts = Array(opts).map { |o| "--#{o}" }
+          opts = xpopen([*envs, pkgconfig, *opts, *args], err:[:child, :out], &:read)
+          Logging.open {puts opts.each_line.map{|s|"=> #{s.inspect}"}}
+          opts.strip if $?.success?
+        }
+      end
+      orig_ldflags = $LDFLAGS
+      if get and !options.empty?
+        get[options]
+      elsif get and try_ldflags(ldflags = get['libs'])
+        if incflags = get['cflags-only-I']
+          $INCFLAGS << " " << incflags
+          cflags = get['cflags-only-other']
+        else
+          cflags = get['cflags']
+        end
+        libs = get['libs-only-l']
+        if cflags
+          $CFLAGS += " " << cflags
+          $CXXFLAGS += " " << cflags
+        end
+        if libs
+          ldflags = (Shellwords.shellwords(ldflags) - Shellwords.shellwords(libs)).quote.join(" ")
+        else
+          libs, ldflags = Shellwords.shellwords(ldflags).partition {|s| s =~ /-l([^ ]+)/ }.map {|l|l.quote.join(" ")}
+        end
+        $libs += " " << libs
+
+        $LDFLAGS = [orig_ldflags, ldflags].join(' ')
+        Logging::message "package configuration for %s\n", pkg
+        Logging::message "incflags: %s\ncflags: %s\nldflags: %s\nlibs: %s\n\n",
+                         incflags, cflags, ldflags, libs
+        [[incflags, cflags].join(' '), ldflags, libs]
+      else
+        Logging::message "package configuration for %s is not found\n", pkg
+        nil
+      end
     end
   end
 
@@ -1968,13 +1970,14 @@ SRC
 
   def configuration(srcdir)
     mk = []
+    verbose = with_config('verbose') ?  "1" : (CONFIG['MKMF_VERBOSE'] || "0")
     vpath = $VPATH.dup
     CONFIG["hdrdir"] ||= $hdrdir
     mk << %{
 SHELL = /bin/sh
 
 # V=0 quiet, V=1 verbose.  other values don't work.
-V = 0
+V = #{verbose}
 V0 = $(V:0=)
 Q1 = $(V:1=)
 Q = $(Q1:0=@)
@@ -2079,6 +2082,11 @@ sitearch = #{CONFIG['sitearch']}
 ruby_version = #{RbConfig::CONFIG['ruby_version']}
 ruby = #{$ruby.sub(%r[\A#{Regexp.quote(RbConfig::CONFIG['bindir'])}(?=/|\z)]) {'$(bindir)'}}
 RUBY = $(ruby#{sep})
+BUILTRUBY = #{if defined?($builtruby) && $builtruby
+    $builtruby
+  else
+    File.join('$(bindir)', CONFIG["RUBY_INSTALL_NAME"] + CONFIG['EXEEXT'])
+  end}
 ruby_headers = #{headers.join(' ')}
 
 RM = #{config_string('RM', &possible_command) || '$(RUBY) -run -e rm -- -f'}
@@ -2113,7 +2121,7 @@ preload = #{defined?($preload) && $preload ? $preload.join(' ') : ''}
   end
   # :startdoc:
 
-  # creates a stub Makefile.
+  # Creates a stub Makefile.
   #
   def dummy_makefile(srcdir)
     configuration(srcdir) << <<RULES << CLEANINGS
@@ -2284,7 +2292,7 @@ RULES
     RbConfig.expand(srcdir = srcprefix.dup)
 
     ext = ".#{$OBJEXT}"
-    orig_srcs = Dir[File.join(srcdir, "*.{#{SRC_EXT.join(%q{,})}}")].sort
+    orig_srcs = Dir[File.join(srcdir, "*.{#{SRC_EXT.join(%q{,})}}")]
     if not $objs
       srcs = $srcs || orig_srcs
       $objs = []
@@ -2294,7 +2302,7 @@ RULES
         h
       }
       unless objs.delete_if {|b, f| f.size == 1}.empty?
-        dups = objs.sort.map {|b, f|
+        dups = objs.map {|b, f|
           "#{b[/.*\./]}{#{f.collect {|n| n[/([^.]+)\z/]}.join(',')}}"
         }
         abort "source files duplication - #{dups.join(", ")}"
@@ -2376,11 +2384,19 @@ TIMESTAMP_DIR = #{$extout && $extmk ? '$(extout)/.timestamp' : '.'}
     install_dirs.each {|d| conf << ("%-14s= %s\n" % d) if /^[[:upper:]]/ =~ d[0]}
     sodir = $extout ? '$(TARGET_SO_DIR)' : '$(RUBYARCHDIR)'
     n = '$(TARGET_SO_DIR)$(TARGET)'
+    cleanobjs = ["$(OBJS)"]
+    if $extmk
+      %w[bc i s].each {|ex| cleanobjs << "$(OBJS:.#{$OBJEXT}=.#{ex})"}
+    end
+    if target
+      config_string('cleanobjs') {|t| cleanobjs << t.gsub(/\$\*/, "$(TARGET)#{deffile ? '-$(arch)': ''}")}
+    end
     conf << "\
 TARGET_SO_DIR =#{$extout ? " $(RUBYARCHDIR)/" : ''}
 TARGET_SO     = $(TARGET_SO_DIR)$(DLLIB)
 CLEANLIBS     = #{'$(TARGET_SO) ' if target}#{config_string('cleanlibs') {|t| t.gsub(/\$\*/) {n}}}
-CLEANOBJS     = *.#{$OBJEXT} #{config_string('cleanobjs') {|t| t.gsub(/\$\*/, "$(TARGET)#{deffile ? '-$(arch)': ''}")} if target} *.bak
+CLEANOBJS     = #{cleanobjs.join(' ')} *.bak
+TARGET_SO_DIR_TIMESTAMP = #{timestamp_file(sodir, target_prefix)}
 " #"
 
     conf = yield(conf) if block_given?
@@ -2414,7 +2430,7 @@ static: #{$extmk && !$static ? "all" : "$(STATIC_LIB)#{$extout ? " install-rb" :
     if target
       f = "$(DLLIB)"
       dest = "$(TARGET_SO)"
-      stamp = timestamp_file(dir, target_prefix)
+      stamp = '$(TARGET_SO_DIR_TIMESTAMP)'
       if $extout
         mfile.puts dest
         mfile.print "clean-so::\n"
@@ -2483,7 +2499,9 @@ static: #{$extmk && !$static ? "all" : "$(STATIC_LIB)#{$extout ? " install-rb" :
         end
       end
     end
-    dirs.unshift(sodir) if target and !dirs.include?(sodir)
+    if target and !dirs.include?(sodir)
+      mfile.print "$(TARGET_SO_DIR_TIMESTAMP):\n\t$(Q) $(MAKEDIRS) $(@D) #{sodir}\n\t$(Q) $(TOUCH) $@\n"
+    end
     dirs.each do |d|
       t = timestamp_file(d, target_prefix)
       mfile.print "#{t}:\n\t$(Q) $(MAKEDIRS) $(@D) #{d}\n\t$(Q) $(TOUCH) $@\n"
@@ -2527,7 +2545,7 @@ site-install-rb: install-rb
     mfile.print "$(TARGET_SO): "
     mfile.print "$(DEFFILE) " if makedef
     mfile.print "$(OBJS) Makefile"
-    mfile.print " #{timestamp_file(sodir, target_prefix)}" if $extout
+    mfile.print " $(TARGET_SO_DIR_TIMESTAMP)" if $extout
     mfile.print "\n"
     mfile.print "\t$(ECHO) linking shared-object #{target_prefix.sub(/\A\/(.*)/, '\1/')}$(DLLIB)\n"
     mfile.print "\t-$(Q)$(RM) $(@#{sep})\n"
@@ -2575,7 +2593,7 @@ site-install-rb: install-rb
 
     if $warnflags = CONFIG['warnflags'] and CONFIG['GCC'] == 'yes'
       # turn warnings into errors only for bundled extensions.
-      config['warnflags'] = $warnflags.gsub(/(\A|\s)-Werror[-=]/, '\1-W')
+      config['warnflags'] = $warnflags.gsub(/(?:\A|\s)-W\Kerror[-=](?!implicit-function-declaration)/, '')
       if /icc\z/ =~ config['CC']
         config['warnflags'].gsub!(/(\A|\s)-W(?:division-by-zero|deprecated-declarations)/, '\1')
       end
@@ -2597,6 +2615,7 @@ site-install-rb: install-rb
     $INCFLAGS << " -I$(hdrdir)/ruby/backward" unless $extmk
     $INCFLAGS << " -I$(hdrdir) -I$(srcdir)"
     $DLDFLAGS = with_config("dldflags", arg_config("DLDFLAGS", config["DLDFLAGS"])).dup
+    config_string("ADDITIONAL_DLDFLAGS") {|flags| $DLDFLAGS << " " << flags} unless $extmk
     $LIBEXT = config['LIBEXT'].dup
     $OBJEXT = config["OBJEXT"].dup
     $EXEEXT = config["EXEEXT"].dup
@@ -2685,7 +2704,7 @@ MESSAGE
   when $mswin
     $nmake = ?m if /nmake/i =~ make
   end
-  $ignore_error = $nmake ? '' : ' 2> /dev/null || true'
+  $ignore_error = " 2> #{File::NULL} || #{$mswin ? 'exit /b0' : 'true'}"
 
   RbConfig::CONFIG["srcdir"] = CONFIG["srcdir"] =
     $srcdir = arg_config("--srcdir", File.dirname($0))
