@@ -61,9 +61,9 @@ exported_object_registry_free(void *ptr)
 const rb_data_type_t rb_memory_view_exported_object_registry_data_type = {
     "memory_view/exported_object_registry",
     {
-	exported_object_registry_mark,
-	exported_object_registry_free,
-	0,
+        exported_object_registry_mark,
+        exported_object_registry_free,
+        0,
     },
     0, 0, RUBY_TYPED_FREE_IMMEDIATELY
 };
@@ -120,9 +120,9 @@ static ID id_memory_view;
 static const rb_data_type_t memory_view_entry_data_type = {
     "memory_view/entry",
     {
-	0,
-	0,
-	0,
+        0,
+        0,
+        0,
     },
     0, 0, RUBY_TYPED_FREE_IMMEDIATELY
 };
@@ -784,7 +784,7 @@ lookup_memory_view_entry(VALUE klass)
 {
     VALUE entry_obj = rb_ivar_lookup(klass, id_memory_view, Qnil);
     while (NIL_P(entry_obj)) {
-        klass = rb_class_get_superclass(klass);
+        klass = rb_class_superclass(klass);
 
         if (klass == rb_cBasicObject || klass == rb_cObject)
             return NULL;
