@@ -7,15 +7,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef _WIN32
+# error This feature is unnecessary on Windows in favor of SxS.
+#endif
+
 #include "ruby-runner.h"
 
-#ifdef MAKE_MJIT_BUILD_DIR
-const char MJIT_HEADER[] = BUILDDIR "/" MJIT_MIN_HEADER;
-#else
-
-#define STRINGIZE(expr) STRINGIZE0(expr)
-#define STRINGIZE0(expr) #expr
-
 static void
 insert_env_path(const char *envname, const char *paths, size_t size, int prepend)
 {
@@ -32,12 +29,24 @@ insert_env_path(const char *envname, const char *paths, size_t size, int prepend
         char *e = malloc(size+n+1);
         size_t pos = 0;
         if (prepend) {
+            if (size == n || (size < n && env[size] == PATH_SEP)) {
+                if (strncmp(paths, env, size) == 0) {
+                    free(e);
+                    return;
+                }
+            }
             memcpy(e, paths, pos = size-1);
             e[pos++] = PATH_SEP;
         }
         memcpy(e+pos, env, n);
         pos += n;
         if (!prepend) {
+            if (size == n || (size < n && env[n-size-1] == PATH_SEP)) {
+                if (strncmp(paths, &env[n-size], size) == 0) {
+                    free(e);
+                    return;
+                }
+            }
             e[pos++] = PATH_SEP;
             memcpy(e+pos, paths, size-1);
             pos += size-1;
@@ -64,10 +73,6 @@ main(int argc, char **argv)
         PATH_SEPARATOR
         EXTOUT_DIR"/"ARCH
         ;
-#ifndef LOAD_RELATIVE
-    static const char mjit_build_dir[] = BUILDDIR"/mjit_build_dir."SOEXT;
-    struct stat stbuf;
-#endif
     const size_t dirsize = sizeof(builddir);
     const size_t namesize = sizeof(rubypath) - dirsize;
     const char *rubyname = rubypath + dirsize;
@@ -75,12 +80,6 @@ main(int argc, char **argv)
 
     insert_env_path(LIBPATHENV, builddir, dirsize, 1);
     insert_env_path("RUBYLIB", rubylib, sizeof(rubylib), 0);
-#ifndef LOAD_RELATIVE
-    if (PRELOADENV[0] && stat(mjit_build_dir, &stbuf) == 0) {
-        insert_env_path(PRELOADENV, mjit_build_dir, sizeof(mjit_build_dir), 1);
-        setenv("MJIT_SEARCH_BUILD_DIR", "true", 0);
-    }
-#endif
 
     if (!(p = strrchr(arg0, '/'))) p = arg0; else p++;
     if (strlen(p) < namesize - 1) {
@@ -94,5 +93,3 @@ main(int argc, char **argv)
     perror(rubypath);
     return -1;
 }
-
-#endif  /* MAKE_MJIT_BUILD_DIR */
