@@ -3,13 +3,13 @@
 RSpec.describe "post bundle message" do
   before :each do
     gemfile <<-G
-      source "#{file_uri_for(gem_repo1)}"
-      gem "rack"
+      source "https://gem.repo1"
+      gem "myrack"
       gem "activesupport", "2.3.5", :group => [:emo, :test]
       group :test do
         gem "rspec"
       end
-      gem "rack-obama", :group => :obama
+      gem "myrack-obama", :group => :obama
     G
   end
 
@@ -114,27 +114,26 @@ RSpec.describe "post bundle message" do
       end
 
       it "should report a helpful error message" do
-        install_gemfile <<-G, :raise_on_error => false
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+        install_gemfile <<-G, raise_on_error: false
+          source "https://gem.repo1"
+          gem "myrack"
           gem "not-a-gem", :group => :development
         G
         expect(err).to include <<-EOS.strip
-Could not find gem 'not-a-gem' in rubygems repository #{file_uri_for(gem_repo1)}/ or installed locally.
-The source does not contain any versions of 'not-a-gem'
+Could not find gem 'not-a-gem' in rubygems repository https://gem.repo1/ or installed locally.
         EOS
       end
 
       it "should report a helpful error message with reference to cache if available" do
         install_gemfile <<-G
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+          source "https://gem.repo1"
+          gem "myrack"
         G
         bundle :cache
-        expect(bundled_app("vendor/cache/rack-1.0.0.gem")).to exist
-        install_gemfile <<-G, :raise_on_error => false
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+        expect(bundled_app("vendor/cache/myrack-1.0.0.gem")).to exist
+        install_gemfile <<-G, raise_on_error: false
+          source "https://gem.repo1"
+          gem "myrack"
           gem "not-a-gem", :group => :development
         G
         expect(err).to include("Could not find gem 'not-a-gem' in").
@@ -143,7 +142,7 @@ The source does not contain any versions of 'not-a-gem'
     end
   end
 
-  describe "for second bundle install run", :bundler => "< 3" do
+  describe "for second bundle install run", bundler: "< 3" do
     it "without any options" do
       2.times { bundle :install }
       expect(out).to include(bundle_show_message)
@@ -180,25 +179,25 @@ The source does not contain any versions of 'not-a-gem'
 
   describe "for bundle update" do
     it "shows proper messages according to the configured groups" do
-      bundle :update, :all => true
+      bundle :update, all: true
       expect(out).not_to include("Gems in the groups")
       expect(out).to include(bundle_updated_message)
 
       bundle "config set --local without emo"
       bundle :install
-      bundle :update, :all => true
+      bundle :update, all: true
       expect(out).to include("Gems in the group 'emo' were not updated")
       expect(out).to include(bundle_updated_message)
 
       bundle "config set --local without emo test"
       bundle :install
-      bundle :update, :all => true
+      bundle :update, all: true
       expect(out).to include("Gems in the groups 'emo' and 'test' were not updated")
       expect(out).to include(bundle_updated_message)
 
       bundle "config set --local without emo obama test"
       bundle :install
-      bundle :update, :all => true
+      bundle :update, all: true
       expect(out).to include("Gems in the groups 'emo', 'obama' and 'test' were not updated")
       expect(out).to include(bundle_updated_message)
     end
