@@ -2,7 +2,7 @@
 
 RSpec.describe "bundle install with specific platforms" do
   let(:google_protobuf) { <<-G }
-    source "#{file_uri_for(gem_repo2)}"
+    source "https://gem.repo2"
     gem "google-protobuf"
   G
 
@@ -33,7 +33,7 @@ RSpec.describe "bundle install with specific platforms" do
       # simulate lockfile created with old bundler, which only locks for ruby platform
       lockfile <<-L
         GEM
-          remote: #{file_uri_for(gem_repo2)}/
+          remote: https://gem.repo2/
           specs:
             google-protobuf (3.0.0.alpha.5.0.5.1)
 
@@ -55,7 +55,7 @@ RSpec.describe "bundle install with specific platforms" do
     end
   end
 
-  it "understands that a non-platform specific gem in a new lockfile locked only to RUBY doesn't necessarily mean installing the non-specific variant" do
+  it "understands that a non-platform specific gem in a new lockfile locked only to ruby doesn't necessarily mean installing the non-specific variant" do
     simulate_platform "x86_64-darwin-15" do
       setup_multiplatform_gem
 
@@ -66,14 +66,14 @@ RSpec.describe "bundle install with specific platforms" do
 
       gemfile google_protobuf
 
-      checksums = checksums_section_when_existing do |c|
+      checksums = checksums_section_when_enabled do |c|
         c.checksum gem_repo2, "google-protobuf", "3.0.0.alpha.4.0"
       end
 
       # simulate lockfile created with old bundler, which only locks for ruby platform
       lockfile <<-L
         GEM
-          remote: #{file_uri_for(gem_repo2)}/
+          remote: https://gem.repo2/
           specs:
             google-protobuf (3.0.0.alpha.4.0)
 
@@ -97,7 +97,7 @@ RSpec.describe "bundle install with specific platforms" do
       # make sure we're still only locked to ruby
       expect(lockfile).to eq <<~L
         GEM
-          remote: #{file_uri_for(gem_repo2)}/
+          remote: https://gem.repo2/
           specs:
             google-protobuf (3.0.0.alpha.5.0.5.1)
 
@@ -113,7 +113,7 @@ RSpec.describe "bundle install with specific platforms" do
     end
   end
 
-  context "when running on a legacy lockfile locked only to RUBY" do
+  context "when running on a legacy lockfile locked only to ruby" do
     around do |example|
       build_repo4 do
         build_gem "nokogiri", "1.3.10"
@@ -124,14 +124,14 @@ RSpec.describe "bundle install with specific platforms" do
       end
 
       gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri"
       G
 
       lockfile <<-L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.3.10)
 
@@ -148,13 +148,13 @@ RSpec.describe "bundle install with specific platforms" do
       simulate_platform "arm64-darwin-22", &example
     end
 
-    it "still installs the generic RUBY variant if necessary" do
-      bundle "install --verbose", artifice: "compact_index", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo4.to_s }
+    it "still installs the generic ruby variant if necessary" do
+      bundle "install --verbose"
       expect(out).to include("Installing nokogiri 1.3.10")
     end
 
-    it "still installs the generic RUBY variant if necessary, even in frozen mode" do
-      bundle "install --verbose", artifice: "compact_index", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo4.to_s, "BUNDLE_FROZEN" => "true" }
+    it "still installs the generic ruby variant if necessary, even in frozen mode" do
+      bundle "install --verbose", env: { "BUNDLE_FROZEN" => "true" }
       expect(out).to include("Installing nokogiri 1.3.10")
     end
   end
@@ -166,7 +166,7 @@ RSpec.describe "bundle install with specific platforms" do
         build_gem("libv8", "8.4.255.0") {|s| s.platform = "universal-darwin" }
 
         build_gem("mini_racer", "1.0.0") do |s|
-          s.add_runtime_dependency "libv8"
+          s.add_dependency "libv8"
         end
       end
 
@@ -176,14 +176,14 @@ RSpec.describe "bundle install with specific platforms" do
       bundle "config set --local path vendor/bundle", env: { "BUNDLER_VERSION" => "2.1.4" }
 
       gemfile <<-G
-        source "https://localgemserver.test"
+        source "https://gem.repo2"
         gem "libv8"
       G
 
       # simulate lockfile created with old bundler, which only locks for ruby platform
       lockfile <<-L
         GEM
-          remote: https://localgemserver.test/
+          remote: https://gem.repo2/
           specs:
             libv8 (8.4.255.0)
 
@@ -197,10 +197,10 @@ RSpec.describe "bundle install with specific platforms" do
            2.1.4
       L
 
-      bundle "install --verbose", artifice: "compact_index", env: { "BUNDLER_VERSION" => "2.1.4", "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
+      bundle "install --verbose", env: { "BUNDLER_VERSION" => "2.1.4" }
       expect(out).to include("Installing libv8 8.4.255.0 (universal-darwin)")
 
-      bundle "add mini_racer --verbose", artifice: "compact_index", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
+      bundle "add mini_racer --verbose"
       expect(out).to include("Using libv8 8.4.255.0 (universal-darwin)")
     end
   end
@@ -213,14 +213,14 @@ RSpec.describe "bundle install with specific platforms" do
       end
 
       gemfile <<-G
-        source "https://localgemserver.test"
+        source "https://gem.repo4"
         gem "grpc"
       G
 
       # simulate lockfile created with old bundler, which only locks for ruby platform
       lockfile <<-L
         GEM
-          remote: https://localgemserver.test/
+          remote: https://gem.repo4/
           specs:
             grpc (1.50.0)
 
@@ -234,7 +234,7 @@ RSpec.describe "bundle install with specific platforms" do
            #{Bundler::VERSION}
       L
 
-      bundle "install --verbose", artifice: "compact_index_precompiled_before", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo4.to_s }
+      bundle "install --verbose", artifice: "compact_index_precompiled_before"
       expect(out).to include("Installing grpc 1.50.0 (universal-darwin)")
     end
   end
@@ -268,7 +268,7 @@ RSpec.describe "bundle install with specific platforms" do
     git = build_git "pg_array_parser", "1.0"
 
     gemfile <<-G
-      source "#{file_uri_for(gem_repo1)}"
+      source "https://gem.repo1"
       gem "pg_array_parser", :git => "#{lib_path("pg_array_parser-1.0")}"
     G
 
@@ -304,7 +304,7 @@ RSpec.describe "bundle install with specific platforms" do
     simulate_platform "x86_64-darwin-15" do
       setup_multiplatform_gem_with_different_dependencies_per_platform
       install_gemfile <<-G
-        source "#{file_uri_for(gem_repo2)}"
+        source "https://gem.repo2"
         gem "facter"
       G
       allow(Bundler::SharedHelpers).to receive(:find_gemfile).and_return(bundled_app_gemfile)
@@ -358,14 +358,14 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo2)}"
+      source "https://gem.repo2"
 
       gem "sorbet-static", "0.5.6403"
     G
 
     lockfile <<~L
       GEM
-        remote: #{file_uri_for(gem_repo2)}/
+        remote: https://gem.repo2/
         specs:
           sorbet-static (0.5.6403-#{Bundler.local_platform})
 
@@ -389,13 +389,13 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "sorbet-static", "0.5.6433"
     G
 
     error_message = <<~ERROR.strip
-      Could not find gem 'sorbet-static (= 0.5.6433)' with platform 'arm64-darwin-21' in rubygems repository #{file_uri_for(gem_repo4)}/ or installed locally.
+      Could not find gem 'sorbet-static (= 0.5.6433)' with platform 'arm64-darwin-21' in rubygems repository https://gem.repo4/ or installed locally.
 
       The source contains the following gems matching 'sorbet-static (= 0.5.6433)':
         * sorbet-static-0.5.6433-universal-darwin-20
@@ -425,7 +425,7 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "sorbet", "0.5.6433"
     G
@@ -434,7 +434,7 @@ RSpec.describe "bundle install with specific platforms" do
       Could not find compatible versions
 
       Because every version of sorbet depends on sorbet-static = 0.5.6433
-        and sorbet-static = 0.5.6433 could not be found in rubygems repository #{file_uri_for(gem_repo4)}/ or installed locally for any resolution platforms (arm64-darwin-21),
+        and sorbet-static = 0.5.6433 could not be found in rubygems repository https://gem.repo4/ or installed locally for any resolution platforms (arm64-darwin-21),
         sorbet cannot be used.
       So, because Gemfile depends on sorbet = 0.5.6433,
         version solving has failed.
@@ -459,13 +459,13 @@ RSpec.describe "bundle install with specific platforms" do
     expect(err).to include(error_message).once
   end
 
-  it "does not generate a lockfile if RUBY platform is forced and some gem has no RUBY variant available" do
+  it "does not generate a lockfile if ruby platform is forced and some gem has no ruby variant available" do
     build_repo4 do
       build_gem("sorbet-static", "0.5.9889") {|s| s.platform = Gem::Platform.local }
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "sorbet-static", "0.5.9889"
     G
@@ -473,22 +473,22 @@ RSpec.describe "bundle install with specific platforms" do
     bundle "lock", raise_on_error: false, env: { "BUNDLE_FORCE_RUBY_PLATFORM" => "true" }
 
     expect(err).to include <<~ERROR.rstrip
-      Could not find gem 'sorbet-static (= 0.5.9889)' with platform 'ruby' in rubygems repository #{file_uri_for(gem_repo4)}/ or installed locally.
+      Could not find gem 'sorbet-static (= 0.5.9889)' with platform 'ruby' in rubygems repository https://gem.repo4/ or installed locally.
 
       The source contains the following gems matching 'sorbet-static (= 0.5.9889)':
         * sorbet-static-0.5.9889-#{Gem::Platform.local}
     ERROR
   end
 
-  it "automatically fixes the lockfile if RUBY platform is locked and some gem has no RUBY variant available" do
+  it "automatically fixes the lockfile if ruby platform is locked and some gem has no ruby variant available" do
     build_repo4 do
       build_gem("sorbet-static-and-runtime", "0.5.10160") do |s|
-        s.add_runtime_dependency "sorbet", "= 0.5.10160"
-        s.add_runtime_dependency "sorbet-runtime", "= 0.5.10160"
+        s.add_dependency "sorbet", "= 0.5.10160"
+        s.add_dependency "sorbet-runtime", "= 0.5.10160"
       end
 
       build_gem("sorbet", "0.5.10160") do |s|
-        s.add_runtime_dependency "sorbet-static", "= 0.5.10160"
+        s.add_dependency "sorbet-static", "= 0.5.10160"
       end
 
       build_gem("sorbet-runtime", "0.5.10160")
@@ -499,14 +499,14 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "sorbet-static-and-runtime"
     G
 
     lockfile <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           sorbet (0.5.10160)
             sorbet-static (= 0.5.10160)
@@ -528,7 +528,7 @@ RSpec.describe "bundle install with specific platforms" do
 
     bundle "update"
 
-    checksums = checksums_section_when_existing do |c|
+    checksums = checksums_section_when_enabled do |c|
       c.checksum gem_repo4, "sorbet", "0.5.10160"
       c.checksum gem_repo4, "sorbet-runtime", "0.5.10160"
       c.checksum gem_repo4, "sorbet-static", "0.5.10160", Gem::Platform.local
@@ -537,7 +537,7 @@ RSpec.describe "bundle install with specific platforms" do
 
     expect(lockfile).to eq <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           sorbet (0.5.10160)
             sorbet-static (= 0.5.10160)
@@ -558,7 +558,7 @@ RSpec.describe "bundle install with specific platforms" do
     L
   end
 
-  it "automatically fixes the lockfile if both RUBY platform and a more specific platform are locked, and some gem has no RUBY variant available" do
+  it "automatically fixes the lockfile if both ruby platform and a more specific platform are locked, and some gem has no ruby variant available" do
     build_repo4 do
       build_gem "nokogiri", "1.12.0"
       build_gem "nokogiri", "1.12.0" do |s|
@@ -577,21 +577,21 @@ RSpec.describe "bundle install with specific platforms" do
 
     simulate_platform "x86_64-darwin-22" do
       install_gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri"
         gem "sorbet-static"
       G
     end
 
-    checksums = checksums_section_when_existing do |c|
-      c.no_checksum "nokogiri", "1.13.0", "x86_64-darwin"
-      c.no_checksum "sorbet-static", "0.5.10601", "x86_64-darwin"
+    checksums = checksums_section_when_enabled do |c|
+      c.checksum gem_repo4, "nokogiri", "1.13.0", "x86_64-darwin"
+      c.checksum gem_repo4, "sorbet-static", "0.5.10601", "x86_64-darwin"
     end
 
     lockfile <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           nokogiri (1.12.0)
           nokogiri (1.12.0-x86_64-darwin)
@@ -615,7 +615,7 @@ RSpec.describe "bundle install with specific platforms" do
 
     expect(lockfile).to eq <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           nokogiri (1.13.0-x86_64-darwin)
           sorbet-static (0.5.10601-x86_64-darwin)
@@ -632,15 +632,15 @@ RSpec.describe "bundle install with specific platforms" do
     L
   end
 
-  it "automatically fixes the lockfile if only RUBY platform is locked and some gem has no RUBY variant available" do
+  it "automatically fixes the lockfile if only ruby platform is locked and some gem has no ruby variant available" do
     build_repo4 do
       build_gem("sorbet-static-and-runtime", "0.5.10160") do |s|
-        s.add_runtime_dependency "sorbet", "= 0.5.10160"
-        s.add_runtime_dependency "sorbet-runtime", "= 0.5.10160"
+        s.add_dependency "sorbet", "= 0.5.10160"
+        s.add_dependency "sorbet-runtime", "= 0.5.10160"
       end
 
       build_gem("sorbet", "0.5.10160") do |s|
-        s.add_runtime_dependency "sorbet-static", "= 0.5.10160"
+        s.add_dependency "sorbet-static", "= 0.5.10160"
       end
 
       build_gem("sorbet-runtime", "0.5.10160")
@@ -651,14 +651,14 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "sorbet-static-and-runtime"
     G
 
     lockfile <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           sorbet (0.5.10160)
             sorbet-static (= 0.5.10160)
@@ -680,7 +680,7 @@ RSpec.describe "bundle install with specific platforms" do
 
     bundle "update"
 
-    checksums = checksums_section_when_existing do |c|
+    checksums = checksums_section_when_enabled do |c|
       c.checksum gem_repo4, "sorbet", "0.5.10160"
       c.checksum gem_repo4, "sorbet-runtime", "0.5.10160"
       c.checksum gem_repo4, "sorbet-static", "0.5.10160", Gem::Platform.local
@@ -689,7 +689,7 @@ RSpec.describe "bundle install with specific platforms" do
 
     expect(lockfile).to eq <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           sorbet (0.5.10160)
             sorbet-static (= 0.5.10160)
@@ -726,7 +726,7 @@ RSpec.describe "bundle install with specific platforms" do
       end
 
       gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri"
         gem "sorbet-static"
@@ -734,7 +734,7 @@ RSpec.describe "bundle install with specific platforms" do
 
       lockfile <<~L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0-arm-linux)
             nokogiri (1.14.0-x86_64-linux)
@@ -755,14 +755,14 @@ RSpec.describe "bundle install with specific platforms" do
 
       bundle "update"
 
-      checksums = checksums_section_when_existing do |c|
+      checksums = checksums_section_when_enabled do |c|
         c.checksum gem_repo4, "nokogiri", "1.14.0", "x86_64-linux"
         c.checksum gem_repo4, "sorbet-static", "0.5.10696", "x86_64-linux"
       end
 
       expect(lockfile).to eq <<~L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0-x86_64-linux)
             sorbet-static (0.5.10696-x86_64-linux)
@@ -794,12 +794,12 @@ RSpec.describe "bundle install with specific platforms" do
 
       # Make sure sorbet-static-0.5.10549-universal-darwin-21 is installed
       install_gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "sorbet-static", "= 0.5.10549"
       G
 
-      checksums = checksums_section_when_existing do |c|
+      checksums = checksums_section_when_enabled do |c|
         c.checksum gem_repo4, "sorbet-static", "0.5.10549", "universal-darwin-20"
         c.checksum gem_repo4, "sorbet-static", "0.5.10549", "universal-darwin-21"
       end
@@ -807,7 +807,7 @@ RSpec.describe "bundle install with specific platforms" do
       # Make sure the lockfile is missing sorbet-static-0.5.10549-universal-darwin-21
       lockfile <<~L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             sorbet-static (0.5.10549-universal-darwin-20)
 
@@ -823,11 +823,9 @@ RSpec.describe "bundle install with specific platforms" do
 
       bundle "install"
 
-      checksums.no_checksum "sorbet-static", "0.5.10549", "universal-darwin-21"
-
       expect(lockfile).to eq <<~L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             sorbet-static (0.5.10549-universal-darwin-20)
             sorbet-static (0.5.10549-universal-darwin-21)
@@ -844,6 +842,63 @@ RSpec.describe "bundle install with specific platforms" do
     end
   end
 
+  it "automatically fixes the lockfile if locked only to ruby, and some locked specs don't meed locked dependencies" do
+    simulate_platform "x86_64-linux" do
+      build_repo4 do
+        build_gem("ibandit", "0.7.0") do |s|
+          s.add_dependency "i18n", "~> 0.7.0"
+        end
+
+        build_gem("i18n", "0.7.0.beta1")
+        build_gem("i18n", "0.7.0")
+      end
+
+      gemfile <<~G
+        source "https://gem.repo4"
+
+        gem "ibandit", "~> 0.7.0"
+      G
+
+      lockfile <<~L
+        GEM
+          remote: https://gem.repo4/
+          specs:
+            i18n (0.7.0.beta1)
+            ibandit (0.7.0)
+              i18n (~> 0.7.0)
+
+        PLATFORMS
+          ruby
+
+        DEPENDENCIES
+          ibandit (~> 0.7.0)
+
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+
+      bundle "lock --update i18n"
+
+      expect(lockfile).to eq <<~L
+        GEM
+          remote: https://gem.repo4/
+          specs:
+            i18n (0.7.0)
+            ibandit (0.7.0)
+              i18n (~> 0.7.0)
+
+        PLATFORMS
+          ruby
+
+        DEPENDENCIES
+          ibandit (~> 0.7.0)
+
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+    end
+  end
+
   it "does not remove ruby if gems for other platforms, and not present in the lockfile, exist in the Gemfile" do
     build_repo4 do
       build_gem "nokogiri", "1.13.8"
@@ -853,45 +908,21 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "nokogiri"
 
       gem "tzinfo", "~> 1.2", platform: :#{not_local_tag}
     G
 
-    original_lockfile = <<~L
-      GEM
-        remote: #{file_uri_for(gem_repo4)}/
-        specs:
-          nokogiri (1.13.8)
-          nokogiri (1.13.8-#{Gem::Platform.local})
-
-      PLATFORMS
-        #{lockfile_platforms("ruby")}
-
-      DEPENDENCIES
-        nokogiri
-        tzinfo (~> 1.2)
-
-      CHECKSUMS
-
-      BUNDLED WITH
-         #{Bundler::VERSION}
-    L
-
-    lockfile original_lockfile
-
-    bundle "lock --update"
-
-    checksums = checksums_section_when_existing do |c|
-      c.no_checksum "nokogiri", "1.13.8"
-      c.no_checksum "nokogiri", "1.13.8", Gem::Platform.local
+    checksums = checksums_section_when_enabled do |c|
+      c.checksum gem_repo4, "nokogiri", "1.13.8"
+      c.checksum gem_repo4, "nokogiri", "1.13.8", Gem::Platform.local
     end
 
-    updated_lockfile = <<~L
+    original_lockfile = <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           nokogiri (1.13.8)
           nokogiri (1.13.8-#{Gem::Platform.local})
@@ -907,30 +938,80 @@ RSpec.describe "bundle install with specific platforms" do
          #{Bundler::VERSION}
     L
 
-    expect(lockfile).to eq(updated_lockfile)
+    lockfile original_lockfile
+
+    bundle "lock --update"
+
+    expect(lockfile).to eq(original_lockfile)
+  end
+
+  it "does not remove ruby if gems for other platforms, and not present in the lockfile, exist in the Gemfile, and the lockfile only has ruby" do
+    build_repo4 do
+      build_gem "nokogiri", "1.13.8"
+      build_gem "nokogiri", "1.13.8" do |s|
+        s.platform = "arm64-darwin"
+      end
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "nokogiri"
+
+      gem "tzinfo", "~> 1.2", platforms: %i[mingw mswin x64_mingw jruby]
+    G
+
+    checksums = checksums_section_when_enabled do |c|
+      c.checksum gem_repo4, "nokogiri", "1.13.8"
+    end
+
+    original_lockfile = <<~L
+      GEM
+        remote: https://gem.repo4/
+        specs:
+          nokogiri (1.13.8)
+
+      PLATFORMS
+        ruby
+
+      DEPENDENCIES
+        nokogiri
+        tzinfo (~> 1.2)
+      #{checksums}
+      BUNDLED WITH
+         #{Bundler::VERSION}
+    L
+
+    lockfile original_lockfile
+
+    simulate_platform "arm64-darwin-23" do
+      bundle "lock --update"
+    end
+
+    expect(lockfile).to eq(original_lockfile)
   end
 
   it "does not remove ruby when adding a new gem to the Gemfile" do
     build_repo4 do
       build_gem "concurrent-ruby", "1.2.2"
-      build_gem "rack", "3.0.7"
+      build_gem "myrack", "3.0.7"
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "concurrent-ruby"
-      gem "rack"
+      gem "myrack"
     G
 
-    checksums = checksums_section_when_existing do |c|
-      c.no_checksum "concurrent-ruby", "1.2.2"
-      c.no_checksum "rack", "3.0.7"
+    checksums = checksums_section_when_enabled do |c|
+      c.checksum gem_repo4, "concurrent-ruby", "1.2.2"
+      c.checksum gem_repo4, "myrack", "3.0.7"
     end
 
     lockfile <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           concurrent-ruby (1.2.2)
 
@@ -948,17 +1029,17 @@ RSpec.describe "bundle install with specific platforms" do
 
     expect(lockfile).to eq <<~L
       GEM
-        remote: #{file_uri_for(gem_repo4)}/
+        remote: https://gem.repo4/
         specs:
           concurrent-ruby (1.2.2)
-          rack (3.0.7)
+          myrack (3.0.7)
 
       PLATFORMS
         #{lockfile_platforms("ruby", generic_local_platform, defaults: [])}
 
       DEPENDENCIES
         concurrent-ruby
-        rack
+        myrack
       #{checksums}
       BUNDLED WITH
          #{Bundler::VERSION}
@@ -968,10 +1049,8 @@ RSpec.describe "bundle install with specific platforms" do
   it "can fallback to a source gem when platform gems are incompatible with current ruby version" do
     setup_multiplatform_gem_with_source_gem
 
-    source = file_uri_for(gem_repo2)
-
     gemfile <<~G
-      source "#{source}"
+      source "https://gem.repo2"
 
       gem "my-precompiled-gem"
     G
@@ -981,7 +1060,7 @@ RSpec.describe "bundle install with specific platforms" do
     # - A source gem with compatible ruby version
     lockfile <<-L
       GEM
-        remote: #{source}/
+        remote: https://gem.repo2/
         specs:
           my-precompiled-gem (3.0.0)
           my-precompiled-gem (3.0.0-#{Bundler.local_platform})
@@ -1017,18 +1096,18 @@ RSpec.describe "bundle install with specific platforms" do
       end
 
       gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri", "1.14.0"
       G
 
-      checksums = checksums_section_when_existing do |c|
+      checksums = checksums_section_when_enabled do |c|
         c.checksum gem_repo4, "nokogiri", "1.14.0", "x86_64-linux"
       end
 
       lockfile <<~L
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0-x86_64-linux)
 
@@ -1044,13 +1123,13 @@ RSpec.describe "bundle install with specific platforms" do
 
       bundle :install
 
-      checksums = checksums_section_when_existing do |c|
+      checksums = checksums_section_when_enabled do |c|
         c.checksum gem_repo4, "nokogiri", "1.14.0"
       end
 
       expect(lockfile).to eq(<<~L)
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0)
 
@@ -1066,6 +1145,60 @@ RSpec.describe "bundle install with specific platforms" do
     end
   end
 
+  it "automatically fixes the lockfile when only ruby platform locked, and adding a dependency with subdependencies not valid for ruby" do
+    simulate_platform "x86_64-linux" do
+      build_repo4 do
+        build_gem("sorbet", "0.5.10160") do |s|
+          s.add_dependency "sorbet-static", "= 0.5.10160"
+        end
+
+        build_gem("sorbet-static", "0.5.10160") do |s|
+          s.platform = "x86_64-linux"
+        end
+      end
+
+      gemfile <<~G
+        source "https://gem.repo4"
+
+        gem "sorbet"
+      G
+
+      lockfile <<~L
+        GEM
+          remote: https://gem.repo4/
+          specs:
+
+        PLATFORMS
+          ruby
+
+        DEPENDENCIES
+
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+
+      bundle "lock"
+
+      expect(lockfile).to eq <<~L
+        GEM
+          remote: https://gem.repo4/
+          specs:
+            sorbet (0.5.10160)
+              sorbet-static (= 0.5.10160)
+            sorbet-static (0.5.10160-x86_64-linux)
+
+        PLATFORMS
+          x86_64-linux
+
+        DEPENDENCIES
+          sorbet
+
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+    end
+  end
+
   it "locks specific platforms automatically" do
     simulate_platform "x86_64-linux" do
       build_repo4 do
@@ -1092,23 +1225,23 @@ RSpec.describe "bundle install with specific platforms" do
       end
 
       gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri"
       G
 
       bundle "lock"
 
-      checksums = checksums_section_when_existing do |c|
-        c.no_checksum "nokogiri", "1.14.0"
-        c.no_checksum "nokogiri", "1.14.0", "arm-linux"
-        c.no_checksum "nokogiri", "1.14.0", "x86_64-linux"
+      checksums = checksums_section_when_enabled do |c|
+        c.checksum gem_repo4, "nokogiri", "1.14.0"
+        c.checksum gem_repo4, "nokogiri", "1.14.0", "arm-linux"
+        c.checksum gem_repo4, "nokogiri", "1.14.0", "x86_64-linux"
       end
 
       # locks all compatible platforms, excluding Java and Windows
       expect(lockfile).to eq(<<~L)
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0)
             nokogiri (1.14.0-arm-linux)
@@ -1127,7 +1260,7 @@ RSpec.describe "bundle install with specific platforms" do
       L
 
       gemfile <<~G
-        source "#{file_uri_for(gem_repo4)}"
+        source "https://gem.repo4"
 
         gem "nokogiri"
         gem "sorbet-static"
@@ -1138,13 +1271,13 @@ RSpec.describe "bundle install with specific platforms" do
       bundle "lock"
 
       checksums.delete "nokogiri", "arm-linux"
-      checksums.no_checksum "sorbet-static", "0.5.10696", "universal-darwin-22"
-      checksums.no_checksum "sorbet-static", "0.5.10696", "x86_64-linux"
+      checksums.checksum gem_repo4, "sorbet-static", "0.5.10696", "universal-darwin-22"
+      checksums.checksum gem_repo4, "sorbet-static", "0.5.10696", "x86_64-linux"
 
       # locks only platforms compatible with all gems in the bundle
       expect(lockfile).to eq(<<~L)
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.14.0)
             nokogiri (1.14.0-x86_64-linux)
@@ -1182,25 +1315,25 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "nokogiri"
       gem "sass-embedded"
     G
 
-    checksums = checksums_section_when_existing do |c|
+    checksums = checksums_section_when_enabled do |c|
       c.checksum gem_repo4, "nokogiri", "1.15.5"
-      c.no_checksum "sass-embedded", "1.69.5"
+      c.checksum gem_repo4, "sass-embedded", "1.69.5"
       c.checksum gem_repo4, "sass-embedded", "1.69.5", "x86_64-linux-gnu"
     end
 
     simulate_platform "x86_64-linux" do
-      bundle "install --verbose", artifice: "compact_index", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo4.to_s }
+      bundle "install --verbose"
 
       # locks all compatible platforms, excluding Java and Windows
       expect(lockfile).to eq(<<~L)
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.15.5)
             sass-embedded (1.69.5)
@@ -1232,21 +1365,21 @@ RSpec.describe "bundle install with specific platforms" do
     end
 
     gemfile <<~G
-      source "#{file_uri_for(gem_repo4)}"
+      source "https://gem.repo4"
 
       gem "nokogiri"
     G
 
-    checksums = checksums_section_when_existing do |c|
+    checksums = checksums_section_when_enabled do |c|
       c.checksum gem_repo4, "nokogiri", "1.15.5", "x86_64-linux"
     end
 
     simulate_platform "x86_64-linux" do
-      bundle "install --verbose", artifice: "compact_index", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo4.to_s }
+      bundle "install --verbose"
 
       expect(lockfile).to eq(<<~L)
         GEM
-          remote: #{file_uri_for(gem_repo4)}/
+          remote: https://gem.repo4/
           specs:
             nokogiri (1.15.5-x86_64-linux)
 
@@ -1262,6 +1395,268 @@ RSpec.describe "bundle install with specific platforms" do
     end
   end
 
+  ["x86_64-linux", "x86_64-linux-musl"].each do |host_platform|
+    describe "on host platform #{host_platform}" do
+      it "adds current musl platform" do
+        build_repo4 do
+          build_gem "rcee_precompiled", "0.5.0" do |s|
+            s.platform = "x86_64-linux"
+          end
+
+          build_gem "rcee_precompiled", "0.5.0" do |s|
+            s.platform = "x86_64-linux-musl"
+          end
+        end
+
+        gemfile <<~G
+          source "https://gem.repo4"
+
+          gem "rcee_precompiled", "0.5.0"
+        G
+
+        simulate_platform host_platform do
+          bundle "lock"
+
+          checksums = checksums_section_when_enabled do |c|
+            c.checksum gem_repo4, "rcee_precompiled", "0.5.0", "x86_64-linux"
+            c.checksum gem_repo4, "rcee_precompiled", "0.5.0", "x86_64-linux-musl"
+          end
+
+          expect(lockfile).to eq(<<~L)
+            GEM
+              remote: https://gem.repo4/
+              specs:
+                rcee_precompiled (0.5.0-x86_64-linux)
+                rcee_precompiled (0.5.0-x86_64-linux-musl)
+
+            PLATFORMS
+              x86_64-linux
+              x86_64-linux-musl
+
+            DEPENDENCIES
+              rcee_precompiled (= 0.5.0)
+            #{checksums}
+            BUNDLED WITH
+               #{Bundler::VERSION}
+          L
+        end
+      end
+    end
+  end
+
+  it "adds current musl platform, when there are also gnu variants", rubygems: ">= 3.3.21" do
+    build_repo4 do
+      build_gem "rcee_precompiled", "0.5.0" do |s|
+        s.platform = "x86_64-linux-gnu"
+      end
+
+      build_gem "rcee_precompiled", "0.5.0" do |s|
+        s.platform = "x86_64-linux-musl"
+      end
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "rcee_precompiled", "0.5.0"
+    G
+
+    simulate_platform "x86_64-linux-musl" do
+      bundle "lock"
+
+      checksums = checksums_section_when_enabled do |c|
+        c.checksum gem_repo4, "rcee_precompiled", "0.5.0", "x86_64-linux-gnu"
+        c.checksum gem_repo4, "rcee_precompiled", "0.5.0", "x86_64-linux-musl"
+      end
+
+      expect(lockfile).to eq(<<~L)
+        GEM
+          remote: https://gem.repo4/
+          specs:
+            rcee_precompiled (0.5.0-x86_64-linux-gnu)
+            rcee_precompiled (0.5.0-x86_64-linux-musl)
+
+        PLATFORMS
+          x86_64-linux-gnu
+          x86_64-linux-musl
+
+        DEPENDENCIES
+          rcee_precompiled (= 0.5.0)
+        #{checksums}
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+    end
+  end
+
+  it "does not add current platform if there's an equivalent less specific platform among the ones resolved" do
+    build_repo4 do
+      build_gem "rcee_precompiled", "0.5.0" do |s|
+        s.platform = "universal-darwin"
+      end
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "rcee_precompiled", "0.5.0"
+    G
+
+    simulate_platform "x86_64-darwin-15" do
+      bundle "lock"
+
+      checksums = checksums_section_when_enabled do |c|
+        c.checksum gem_repo4, "rcee_precompiled", "0.5.0", "universal-darwin"
+      end
+
+      expect(lockfile).to eq(<<~L)
+        GEM
+          remote: https://gem.repo4/
+          specs:
+            rcee_precompiled (0.5.0-universal-darwin)
+
+        PLATFORMS
+          universal-darwin
+
+        DEPENDENCIES
+          rcee_precompiled (= 0.5.0)
+        #{checksums}
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+    end
+  end
+
+  it "does not re-resolve when a specific platform, but less specific than the current platform, is locked" do
+    build_repo4 do
+      build_gem "nokogiri"
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "nokogiri"
+    G
+
+    lockfile <<~L
+      GEM
+        remote: https://gem.repo4/
+        specs:
+          nokogiri (1.0)
+
+      PLATFORMS
+        arm64-darwin
+
+      DEPENDENCIES
+        nokogiri!
+
+      BUNDLED WITH
+         #{Bundler::VERSION}
+    L
+
+    simulate_platform "arm64-darwin-23" do
+      bundle "install --verbose"
+
+      expect(out).to include("Found no changes, using resolution from the lockfile")
+    end
+  end
+
+  it "does not remove generic platform gems locked for a specific platform from lockfile when unlocking an unrelated gem" do
+    build_repo4 do
+      build_gem "ffi"
+
+      build_gem "ffi" do |s|
+        s.platform = "x86_64-linux"
+      end
+
+      build_gem "nokogiri"
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "ffi"
+      gem "nokogiri"
+    G
+
+    original_lockfile = <<~L
+      GEM
+        remote: https://gem.repo4/
+        specs:
+          ffi (1.0)
+          nokogiri (1.0)
+
+      PLATFORMS
+        x86_64-linux
+
+      DEPENDENCIES
+        ffi
+        nokogiri
+
+      BUNDLED WITH
+         #{Bundler::VERSION}
+    L
+
+    lockfile original_lockfile
+
+    simulate_platform "x86_64-linux" do
+      bundle "lock --update nokogiri"
+
+      expect(lockfile).to eq(original_lockfile)
+    end
+  end
+
+  it "does not remove generic platform gems locked for a specific platform from lockfile when unlocking an unrelated gem, and variants for other platform also locked" do
+    build_repo4 do
+      build_gem "ffi"
+
+      build_gem "ffi" do |s|
+        s.platform = "x86_64-linux"
+      end
+
+      build_gem "ffi" do |s|
+        s.platform = "java"
+      end
+
+      build_gem "nokogiri"
+    end
+
+    gemfile <<~G
+      source "https://gem.repo4"
+
+      gem "ffi"
+      gem "nokogiri"
+    G
+
+    original_lockfile = <<~L
+      GEM
+        remote: https://gem.repo4/
+        specs:
+          ffi (1.0)
+          ffi (1.0-java)
+          nokogiri (1.0)
+
+      PLATFORMS
+        java
+        x86_64-linux
+
+      DEPENDENCIES
+        ffi
+        nokogiri
+
+      BUNDLED WITH
+         #{Bundler::VERSION}
+    L
+
+    lockfile original_lockfile
+
+    simulate_platform "x86_64-linux" do
+      bundle "lock --update nokogiri"
+
+      expect(lockfile).to eq(original_lockfile)
+    end
+  end
+
   private
 
   def setup_multiplatform_gem
@@ -1287,7 +1682,7 @@ RSpec.describe "bundle install with specific platforms" do
       build_gem("facter", "2.4.6")
       build_gem("facter", "2.4.6") do |s|
         s.platform = "universal-darwin"
-        s.add_runtime_dependency "CFPropertyList"
+        s.add_dependency "CFPropertyList"
       end
       build_gem("CFPropertyList")
     end
