@@ -1,8 +1,6 @@
 # frozen_string_literal: true
 
-require File.expand_path("../compact_index", __FILE__)
-
-Artifice.deactivate
+require_relative "helpers/compact_index"
 
 class CompactIndexRedirect < CompactIndexAPI
   get "/fetch/actual/gem/:id" do
@@ -18,4 +16,6 @@ class CompactIndexRedirect < CompactIndexAPI
   end
 end
 
+require_relative "helpers/artifice"
+
 Artifice.activate_with(CompactIndexRedirect)
