@@ -14,7 +14,7 @@ describe "Enumerator#rewind" do
   end
 
   it "returns self" do
-    @enum.rewind.should == @enum
+    @enum.rewind.should.equal? @enum
   end
 
   it "has no effect on a new enumerator" do
@@ -49,7 +49,7 @@ describe "Enumerator#rewind" do
     obj = mock('rewinder')
     enum = obj.to_enum
     obj.should_receive(:each).at_most(1)
-    -> { enum.rewind.should == enum }.should_not raise_error
+    enum.rewind.should == enum
   end
 end
 
