@@ -14,10 +14,8 @@ describe "StopIteration#result" do
   it "returns the method-returned-object from an Enumerator" do
     @enum.next
     @enum.next
-    -> { @enum.next }.should(
-      raise_error(StopIteration) do |error|
-        error.result.should equal(:method_returned)
-      end
-    )
+    -> { @enum.next }.should raise_error(StopIteration) { |error|
+      error.result.should equal(:method_returned)
+    }
   end
 end
