@@ -17,10 +17,8 @@ describe "MatchData#post_match" do
     str.match(/c/).post_match.encoding.should equal(Encoding::ISO_8859_1)
   end
 
-  ruby_version_is "3.0" do
-    it "returns an instance of String when given a String subclass" do
-      str = MatchDataSpecs::MyString.new("THX1138: The Movie")
-      /(.)(.)(\d+)(\d)/.match(str).post_match.should be_an_instance_of(String)
-    end
+  it "returns an instance of String when given a String subclass" do
+    str = MatchDataSpecs::MyString.new("THX1138: The Movie")
+    /(.)(.)(\d+)(\d)/.match(str).post_match.should be_an_instance_of(String)
   end
 end
