@@ -72,19 +72,9 @@ describe "Module#attr_writer" do
     Module.should have_public_instance_method(:attr_writer, false)
   end
 
-  ruby_version_is ""..."3.0" do
-    it "returns nil" do
-      Class.new do
-        (attr_writer :foo, 'bar').should == nil
-      end
-    end
-  end
-
-  ruby_version_is "3.0" do
-    it "returns an array of defined method names as symbols" do
-      Class.new do
-        (attr_writer :foo, 'bar').should == [:foo=, :bar=]
-      end
+  it "returns an array of defined method names as symbols" do
+    Class.new do
+      (attr_writer :foo, 'bar').should == [:foo=, :bar=]
     end
   end
 end
