@@ -14,26 +14,16 @@ describe "Random::DEFAULT" do
       seed1.should != seed2
     end
 
-    ruby_version_is ''...'3.0' do
-      it "returns a Random instance" do
-        suppress_warning do
-          Random::DEFAULT.should be_an_instance_of(Random)
-        end
+    it "refers to the Random class" do
+      suppress_warning do
+        Random::DEFAULT.should.equal?(Random)
       end
     end
 
-    ruby_version_is '3.0' do
-      it "refers to the Random class" do
-        suppress_warning do
-          Random::DEFAULT.should.equal?(Random)
-        end
-      end
-
-      it "is deprecated" do
-        -> {
-          Random::DEFAULT.should.equal?(Random)
-        }.should complain(/constant Random::DEFAULT is deprecated/)
-      end
+    it "is deprecated" do
+      -> {
+        Random::DEFAULT.should.equal?(Random)
+      }.should complain(/constant Random::DEFAULT is deprecated/)
     end
   end
 
