@@ -204,6 +204,13 @@ describe "String#rindex with String" do
   it "handles a substring in a subset encoding" do
     'été'.rindex('t'.force_encoding(Encoding::US_ASCII)).should == 1
   end
+
+  it "raises an Encoding::CompatibilityError if the encodings are incompatible" do
+    str = 'abc'.force_encoding("ISO-2022-JP")
+    pattern = 'b'.force_encoding("EUC-JP")
+
+    -> { str.rindex(pattern) }.should raise_error(Encoding::CompatibilityError, "incompatible character encodings: ISO-2022-JP and EUC-JP")
+  end
 end
 
 describe "String#rindex with Regexp" do
@@ -265,15 +272,6 @@ describe "String#rindex with Regexp" do
     $~.should == nil
   end
 
-  it "always clear $~" do
-    "a".rindex(/a/)
-    $~.should_not == nil
-
-    string = "blablabla"
-    string.rindex(/bla/, -(string.length + 1))
-    $~.should == nil
-  end
-
   it "starts the search at the given offset" do
     "blablabla".rindex(/.{0}/, 5).should == 5
     "blablabla".rindex(/.{1}/, 5).should == 5
@@ -382,6 +380,6 @@ describe "String#rindex with Regexp" do
     re = Regexp.new "れ".encode(Encoding::EUC_JP)
     -> do
       "あれ".rindex re
-    end.should raise_error(Encoding::CompatibilityError)
+    end.should raise_error(Encoding::CompatibilityError, "incompatible encoding regexp match (EUC-JP regexp with UTF-8 string)")
   end
 end
