@@ -1,4 +1,4 @@
-require File.expand_path('../../../../spec_helper', __FILE__)
+require_relative '../../../spec_helper'
 require 'cgi'
 
 describe "CGI::QueryExtension#referer" do
