@@ -8,14 +8,14 @@ describe "Addrinfo#ipv4_private?" do
     end
 
     it "returns true for a private address" do
-      Addrinfo.ip('10.0.0.0').ipv4_private?.should == true
-      Addrinfo.ip('10.0.0.5').ipv4_private?.should == true
+      Addrinfo.ip('10.0.0.0').should.ipv4_private?
+      Addrinfo.ip('10.0.0.5').should.ipv4_private?
 
-      Addrinfo.ip('172.16.0.0').ipv4_private?.should == true
-      Addrinfo.ip('172.16.0.5').ipv4_private?.should == true
+      Addrinfo.ip('172.16.0.0').should.ipv4_private?
+      Addrinfo.ip('172.16.0.5').should.ipv4_private?
 
-      Addrinfo.ip('192.168.0.0').ipv4_private?.should == true
-      Addrinfo.ip('192.168.0.5').ipv4_private?.should == true
+      Addrinfo.ip('192.168.0.0').should.ipv4_private?
+      Addrinfo.ip('192.168.0.5').should.ipv4_private?
     end
 
     it "returns false for a public address" do
