@@ -23,17 +23,6 @@ describe "StringIO#reopen when passed [Object, Integer]" do
     @io.string.should == "reopened, another time"
   end
 
-  ruby_version_is ""..."3.0" do
-    # NOTE: WEIRD!
-    it "does not taint self when the passed Object was tainted" do
-      @io.reopen("reopened".taint, IO::RDONLY)
-      @io.tainted?.should be_false
-
-      @io.reopen("reopened".taint, IO::WRONLY)
-      @io.tainted?.should be_false
-    end
-  end
-
   it "tries to convert the passed Object to a String using #to_str" do
     obj = mock("to_str")
     obj.should_receive(:to_str).and_return("to_str")
@@ -92,17 +81,6 @@ describe "StringIO#reopen when passed [Object, Object]" do
     str.should == ""
   end
 
-  ruby_version_is ""..."3.0" do
-    # NOTE: WEIRD!
-    it "does not taint self when the passed Object was tainted" do
-      @io.reopen("reopened".taint, "r")
-      @io.tainted?.should be_false
-
-      @io.reopen("reopened".taint, "w")
-      @io.tainted?.should be_false
-    end
-  end
-
   it "tries to convert the passed Object to a String using #to_str" do
     obj = mock("to_str")
     obj.should_receive(:to_str).and_return("to_str")
@@ -164,14 +142,6 @@ describe "StringIO#reopen when passed [String]" do
     @io.string.should == "reopened"
   end
 
-  ruby_version_is ""..."3.0" do
-    # NOTE: WEIRD!
-    it "does not taint self when the passed Object was tainted" do
-      @io.reopen("reopened".taint)
-      @io.tainted?.should be_false
-    end
-  end
-
   it "resets self's position to 0" do
     @io.read(5)
     @io.reopen("reopened")
