@@ -1,7 +1,6 @@
-require File.expand_path('../../fixtures/classes', __FILE__)
-
+require_relative "../../../spec_helper"
 platform_is :windows do
-  require 'win32ole'
+  require_relative '../fixtures/classes'
 
   describe "WIN32OLE.codepage=" do
     it "sets codepage" do
