@@ -103,11 +103,13 @@
 #ifdef NOT_RUBY
 #include "regint.h"
 #include "st.h"
-#else
+#include <assert.h>
+#elif defined RUBY_EXPORT
 #include "internal.h"
 #include "internal/bits.h"
 #include "internal/hash.h"
 #include "internal/sanitizers.h"
+#include "ruby_assert.h"
 #endif
 
 #include <stdio.h>
@@ -115,7 +117,6 @@
 #include <stdlib.h>
 #endif
 #include <string.h>
-#include <assert.h>
 
 #ifdef __GNUC__
 #define PREFETCH(addr, write_p) __builtin_prefetch(addr, write_p)
@@ -313,15 +314,20 @@ static const struct st_features features[] = {
 #define RESERVED_HASH_VAL (~(st_hash_t) 0)
 #define RESERVED_HASH_SUBSTITUTION_VAL ((st_hash_t) 0)
 
+static inline st_hash_t
+normalize_hash_value(st_hash_t hash)
+{
+    /* RESERVED_HASH_VAL is used for a deleted entry.  Map it into
+       another value.  Such mapping should be extremely rare.  */
+    return hash == RESERVED_HASH_VAL ? RESERVED_HASH_SUBSTITUTION_VAL : hash;
+}
+
 /* Return hash value of KEY for table TAB.  */
 static inline st_hash_t
 do_hash(st_data_t key, st_table *tab)
 {
     st_hash_t hash = (st_hash_t)(tab->type->hash)(key);
-
-    /* RESERVED_HASH_VAL is used for a deleted entry.  Map it into
-       another value.  Such mapping should be extremely rare.  */
-    return hash == RESERVED_HASH_VAL ? RESERVED_HASH_SUBSTITUTION_VAL : hash;
+    return normalize_hash_value(hash);
 }
 
 /* Power of 2 defining the minimal number of allocated entries.  */
@@ -342,7 +348,7 @@ get_power2(st_index_t size)
     unsigned int n = ST_INDEX_BITS - nlz_intptr(size);
     if (n <= MAX_POWER2)
         return n < MINIMAL_POWER2 ? MINIMAL_POWER2 : n;
-#ifndef NOT_RUBY
+#ifdef RUBY
     /* Ran out of the table entries */
     rb_raise(rb_eRuntimeError, "st_table too big");
 #endif
@@ -509,13 +515,9 @@ stat_col(void)
 }
 #endif
 
-/* Create and return table with TYPE which can hold at least SIZE
-   entries.  The real number of entries which the table can hold is
-   the nearest power of two for SIZE.  */
 st_table *
-st_init_table_with_size(const struct st_hash_type *type, st_index_t size)
+st_init_existing_table_with_size(st_table *tab, const struct st_hash_type *type, st_index_t size)
 {
-    st_table *tab;
     int n;
 
 #ifdef HASH_LOG
@@ -536,11 +538,7 @@ st_init_table_with_size(const struct st_hash_type *type, st_index_t size)
     if (n < 0)
         return NULL;
 #endif
-    tab = (st_table *) malloc(sizeof (st_table));
-#ifndef RUBY
-    if (tab == NULL)
-        return NULL;
-#endif
+
     tab->type = type;
     tab->entry_power = n;
     tab->bin_power = features[n].bin_power;
@@ -569,6 +567,30 @@ st_init_table_with_size(const struct st_hash_type *type, st_index_t size)
     return tab;
 }
 
+/* Create and return table with TYPE which can hold at least SIZE
+   entries.  The real number of entries which the table can hold is
+   the nearest power of two for SIZE.  */
+st_table *
+st_init_table_with_size(const struct st_hash_type *type, st_index_t size)
+{
+    st_table *tab = malloc(sizeof(st_table));
+#ifndef RUBY
+    if (tab == NULL)
+        return NULL;
+#endif
+
+#ifdef RUBY
+    st_init_existing_table_with_size(tab, type, size);
+#else
+    if (st_init_existing_table_with_size(tab, type, size) == NULL) {
+        free(tab);
+        return NULL;
+    }
+#endif
+
+    return tab;
+}
+
 size_t
 st_table_size(const struct st_table *tbl)
 {
@@ -641,8 +663,7 @@ st_clear(st_table *tab)
 void
 st_free_table(st_table *tab)
 {
-    if (tab->bins != NULL)
-        free(tab->bins);
+    free(tab->bins);
     free(tab->entries);
     free(tab);
 }
@@ -764,12 +785,14 @@ rebuild_table_with(st_table *new_tab, st_table *tab)
         new_tab->num_entries++;
         ni++;
     }
+
+    assert(new_tab->num_entries == tab->num_entries);
+
     if (new_tab != tab) {
         tab->entry_power = new_tab->entry_power;
         tab->bin_power = new_tab->bin_power;
         tab->size_ind = new_tab->size_ind;
-        if (tab->bins != NULL)
-            free(tab->bins);
+        free(tab->bins);
         tab->bins = new_tab->bins;
         free(tab->entries);
         tab->entries = new_tab->entries;
@@ -781,7 +804,7 @@ rebuild_table_with(st_table *new_tab, st_table *tab)
 }
 
 /* Return the next secondary hash index for table TAB using previous
-   index IND and PERTERB.  Finally modulo of the function becomes a
+   index IND and PERTURB.  Finally modulo of the function becomes a
    full *cycle linear congruential generator*, in other words it
    guarantees traversing all table bins in extreme case.
 
@@ -793,10 +816,10 @@ rebuild_table_with(st_table *new_tab, st_table *tab)
 
    For our case a is 5, c is 1, and m is a power of two.  */
 static inline st_index_t
-secondary_hash(st_index_t ind, st_table *tab, st_index_t *perterb)
+secondary_hash(st_index_t ind, st_table *tab, st_index_t *perturb)
 {
-    *perterb >>= 11;
-    ind = (ind << 2) + ind + *perterb + 1;
+    *perturb >>= 11;
+    ind = (ind << 2) + ind + *perturb + 1;
     return hash_bin(ind, tab);
 }
 
@@ -839,7 +862,7 @@ find_table_entry_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     st_index_t d;
 #else
-    st_index_t peterb;
+    st_index_t perturb;
 #endif
     st_index_t bin;
     st_table_entry *entries = tab->entries;
@@ -848,7 +871,7 @@ find_table_entry_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     d = 1;
 #else
-    peterb = hash_value;
+    perturb = hash_value;
 #endif
     FOUND_BIN;
     for (;;) {
@@ -866,7 +889,7 @@ find_table_entry_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
         ind = hash_bin(ind + d, tab);
         d++;
 #else
-        ind = secondary_hash(ind, tab, &peterb);
+        ind = secondary_hash(ind, tab, &perturb);
 #endif
         COLLISION;
     }
@@ -885,7 +908,7 @@ find_table_bin_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     st_index_t d;
 #else
-    st_index_t peterb;
+    st_index_t perturb;
 #endif
     st_index_t bin;
     st_table_entry *entries = tab->entries;
@@ -894,7 +917,7 @@ find_table_bin_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     d = 1;
 #else
-    peterb = hash_value;
+    perturb = hash_value;
 #endif
     FOUND_BIN;
     for (;;) {
@@ -912,7 +935,7 @@ find_table_bin_ind(st_table *tab, st_hash_t hash_value, st_data_t key)
         ind = hash_bin(ind + d, tab);
         d++;
 #else
-        ind = secondary_hash(ind, tab, &peterb);
+        ind = secondary_hash(ind, tab, &perturb);
 #endif
         COLLISION;
     }
@@ -929,7 +952,7 @@ find_table_bin_ind_direct(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     st_index_t d;
 #else
-    st_index_t peterb;
+    st_index_t perturb;
 #endif
     st_index_t bin;
 
@@ -937,7 +960,7 @@ find_table_bin_ind_direct(st_table *tab, st_hash_t hash_value, st_data_t key)
 #ifdef QUADRATIC_PROBE
     d = 1;
 #else
-    peterb = hash_value;
+    perturb = hash_value;
 #endif
     FOUND_BIN;
     for (;;) {
@@ -948,7 +971,7 @@ find_table_bin_ind_direct(st_table *tab, st_hash_t hash_value, st_data_t key)
         ind = hash_bin(ind + d, tab);
         d++;
 #else
-        ind = secondary_hash(ind, tab, &peterb);
+        ind = secondary_hash(ind, tab, &perturb);
 #endif
         COLLISION;
     }
@@ -973,7 +996,7 @@ find_table_bin_ptr_and_reserve(st_table *tab, st_hash_t *hash_value,
 #ifdef QUADRATIC_PROBE
     st_index_t d;
 #else
-    st_index_t peterb;
+    st_index_t perturb;
 #endif
     st_index_t entry_index;
     st_index_t first_deleted_bin_ind;
@@ -983,7 +1006,7 @@ find_table_bin_ptr_and_reserve(st_table *tab, st_hash_t *hash_value,
 #ifdef QUADRATIC_PROBE
     d = 1;
 #else
-    peterb = curr_hash_value;
+    perturb = curr_hash_value;
 #endif
     FOUND_BIN;
     first_deleted_bin_ind = UNDEFINED_BIN_IND;
@@ -1013,7 +1036,7 @@ find_table_bin_ptr_and_reserve(st_table *tab, st_hash_t *hash_value,
         ind = hash_bin(ind + d, tab);
         d++;
 #else
-        ind = secondary_hash(ind, tab, &peterb);
+        ind = secondary_hash(ind, tab, &perturb);
 #endif
         COLLISION;
     }
@@ -1146,6 +1169,8 @@ st_add_direct_with_hash(st_table *tab,
     st_index_t ind;
     st_index_t bin_ind;
 
+    assert(hash != RESERVED_HASH_VAL);
+
     rebuild_table_if_necessary(tab);
     ind = tab->entries_bound++;
     entry = &tab->entries[ind];
@@ -1163,7 +1188,7 @@ void
 rb_st_add_direct_with_hash(st_table *tab,
                            st_data_t key, st_data_t value, st_hash_t hash)
 {
-    st_add_direct_with_hash(tab, key, value, hash);
+    st_add_direct_with_hash(tab, key, value, normalize_hash_value(hash));
 }
 
 /* Insert (KEY, VALUE) into table TAB.  The table should not have
@@ -1226,6 +1251,36 @@ st_insert2(st_table *tab, st_data_t key, st_data_t value,
     return 1;
 }
 
+/* Create a copy of old_tab into new_tab. */
+st_table *
+st_replace(st_table *new_tab, st_table *old_tab)
+{
+    *new_tab = *old_tab;
+    if (old_tab->bins == NULL)
+        new_tab->bins = NULL;
+    else {
+        new_tab->bins = (st_index_t *) malloc(bins_size(old_tab));
+#ifndef RUBY
+        if (new_tab->bins == NULL) {
+            return NULL;
+        }
+#endif
+    }
+    new_tab->entries = (st_table_entry *) malloc(get_allocated_entries(old_tab)
+                                                 * sizeof(st_table_entry));
+#ifndef RUBY
+    if (new_tab->entries == NULL) {
+        return NULL;
+    }
+#endif
+    MEMCPY(new_tab->entries, old_tab->entries, st_table_entry,
+           get_allocated_entries(old_tab));
+    if (old_tab->bins != NULL)
+        MEMCPY(new_tab->bins, old_tab->bins, char, bins_size(old_tab));
+
+    return new_tab;
+}
+
 /* Create and return a copy of table OLD_TAB.  */
 st_table *
 st_copy(st_table *old_tab)
@@ -1237,30 +1292,12 @@ st_copy(st_table *old_tab)
     if (new_tab == NULL)
         return NULL;
 #endif
-    *new_tab = *old_tab;
-    if (old_tab->bins == NULL)
-        new_tab->bins = NULL;
-    else {
-        new_tab->bins = (st_index_t *) malloc(bins_size(old_tab));
-#ifndef RUBY
-        if (new_tab->bins == NULL) {
-            free(new_tab);
-            return NULL;
-        }
-#endif
-    }
-    new_tab->entries = (st_table_entry *) malloc(get_allocated_entries(old_tab)
-                                                 * sizeof(st_table_entry));
-#ifndef RUBY
-    if (new_tab->entries == NULL) {
+
+    if (st_replace(new_tab, old_tab) == NULL) {
         st_free_table(new_tab);
         return NULL;
     }
-#endif
-    MEMCPY(new_tab->entries, old_tab->entries, st_table_entry,
-           get_allocated_entries(old_tab));
-    if (old_tab->bins != NULL)
-        MEMCPY(new_tab->bins, old_tab->bins, char, bins_size(old_tab));
+
     return new_tab;
 }
 
@@ -2059,6 +2096,7 @@ st_numhash(st_data_t n)
     return (st_index_t)((n>>s1|(n<<s2)) ^ (n>>s2));
 }
 
+#ifdef RUBY
 /* Expand TAB to be suitable for holding SIZ entries in total.
    Pre-existing entries remain not deleted inside of TAB, but its bins
    are cleared to expect future reconstruction. See rehash below. */
@@ -2075,10 +2113,8 @@ st_expand_table(st_table *tab, st_index_t siz)
     n = get_allocated_entries(tab);
     MEMCPY(tmp->entries, tab->entries, st_table_entry, n);
     free(tab->entries);
-    if (tab->bins != NULL)
-        free(tab->bins);
-    if (tmp->bins != NULL)
-        free(tmp->bins);
+    free(tab->bins);
+    free(tmp->bins);
     tab->entry_power = tmp->entry_power;
     tab->bin_power = tmp->bin_power;
     tab->size_ind = tmp->size_ind;
@@ -2096,10 +2132,10 @@ st_rehash_linear(st_table *tab)
     int eq_p, rebuilt_p;
     st_index_t i, j;
     st_table_entry *p, *q;
-    if (tab->bins) {
-        free(tab->bins);
-        tab->bins = NULL;
-    }
+
+    free(tab->bins);
+    tab->bins = NULL;
+
     for (i = tab->entries_start; i < tab->entries_bound; i++) {
         p = &tab->entries[i];
         if (DELETED_ENTRY_P(p))
@@ -2140,7 +2176,7 @@ st_rehash_indexed(st_table *tab)
 #ifdef QUADRATIC_PROBE
         st_index_t d = 1;
 #else
-        st_index_t peterb = p->hash;
+        st_index_t perturb = p->hash;
 #endif
 
         if (DELETED_ENTRY_P(p))
@@ -2173,7 +2209,7 @@ st_rehash_indexed(st_table *tab)
                     ind = hash_bin(ind + d, tab);
                     d++;
 #else
-                    ind = secondary_hash(ind, tab, &peterb);
+                    ind = secondary_hash(ind, tab, &perturb);
 #endif
                 }
             }
@@ -2198,7 +2234,6 @@ st_rehash(st_table *tab)
     } while (rebuilt_p);
 }
 
-#ifdef RUBY
 static st_data_t
 st_stringify(VALUE key)
 {
