@@ -549,7 +549,7 @@ module TestNetHTTP_version_1_1_methods
       conn = Net::HTTP.new('localhost', port)
       conn.write_timeout = EnvUtil.apply_timeout_scale(0.01)
       conn.read_timeout = EnvUtil.apply_timeout_scale(0.01) if windows?
-      conn.open_timeout = EnvUtil.apply_timeout_scale(0.1)
+      conn.open_timeout = EnvUtil.apply_timeout_scale(1)
 
       th = Thread.new do
         err = !windows? ? Net::WriteTimeout : Net::ReadTimeout
@@ -573,9 +573,9 @@ module TestNetHTTP_version_1_1_methods
       port = server.addr[1]
 
       conn = Net::HTTP.new('localhost', port)
-      conn.write_timeout = 0.01
-      conn.read_timeout = 0.01 if windows?
-      conn.open_timeout = 0.1
+      conn.write_timeout = EnvUtil.apply_timeout_scale(0.01)
+      conn.read_timeout = EnvUtil.apply_timeout_scale(0.01) if windows?
+      conn.open_timeout = EnvUtil.apply_timeout_scale(1)
 
       req = Net::HTTP::Post.new('/')
       data = "a"*50_000_000
@@ -1249,7 +1249,7 @@ end
 
 class TestNetHTTPLocalBind < Test::Unit::TestCase
   CONFIG = {
-    'host' => 'localhost',
+    'host' => '127.0.0.1',
     'proxy_host' => nil,
     'proxy_port' => nil,
   }
@@ -1286,7 +1286,7 @@ end
 
 class TestNetHTTPForceEncoding < Test::Unit::TestCase
   CONFIG = {
-    'host' => 'localhost',
+    'host' => '127.0.0.1',
     'proxy_host' => nil,
     'proxy_port' => nil,
   }
