@@ -1,5 +1,5 @@
 # frozen_string_literal: true
-require 'rdoc/test_case'
+require_relative 'helper'
 
 class TestRDocMarkup < RDoc::TestCase
 
@@ -93,4 +93,3 @@ the time
   end
 
 end
-
