@@ -1,4 +1,5 @@
-require 'rdoc/test_case'
+# frozen_string_literal: true
+require_relative 'helper'
 
 class TestRDocRd < RDoc::TestCase
 
@@ -27,4 +28,3 @@ class TestRDocRd < RDoc::TestCase
   end
 
 end
-
