@@ -2,7 +2,9 @@
 require 'test/unit'
 
 class TestEnv < Test::Unit::TestCase
-  IGNORE_CASE = /bccwin|mswin|mingw/ =~ RUBY_PLATFORM
+  windows = /bccwin|mswin|mingw/ =~ RUBY_PLATFORM
+  IGNORE_CASE = windows
+  ENCODING = windows ? Encoding::UTF_8 : Encoding.find("locale")
   PATH_ENV = "PATH"
   INVALID_ENVVARS = [
     "foo\0bar",
@@ -69,25 +71,20 @@ class TestEnv < Test::Unit::TestCase
   end
 
   def test_clone
-    warning = /ENV\.clone is deprecated; use ENV\.to_h instead/
-    clone = assert_deprecated_warning(warning) {
+    message = /Cannot clone ENV/
+    assert_raise_with_message(TypeError, message) {
       ENV.clone
     }
-    assert_same(ENV, clone)
-
-    clone = assert_deprecated_warning(warning) {
+    assert_raise_with_message(TypeError, message) {
       ENV.clone(freeze: false)
     }
-    assert_same(ENV, clone)
-
-    clone = assert_deprecated_warning(warning) {
+    assert_raise_with_message(TypeError, message) {
       ENV.clone(freeze: nil)
     }
-    assert_same(ENV, clone)
-
-    assert_raise(TypeError) {
+    assert_raise_with_message(TypeError, message) {
       ENV.clone(freeze: true)
     }
+
     assert_raise(ArgumentError) {
       ENV.clone(freeze: 1)
     }
@@ -358,6 +355,13 @@ class TestEnv < Test::Unit::TestCase
     end
   end
 
+  def test_inspect_encoding
+    ENV.clear
+    key = "VAR\u{e5 e1 e2 e4 e3 101 3042}"
+    ENV[key] = "foo"
+    assert_equal(%{{#{(key.encode(ENCODING) rescue key.b).inspect}=>"foo"}}, ENV.inspect)
+  end
+
   def test_to_a
     ENV.clear
     ENV["foo"] = "bar"
@@ -408,8 +412,7 @@ class TestEnv < Test::Unit::TestCase
       assert_equal("foo", v)
     end
     assert_invalid_env {|var| ENV.assoc(var)}
-    encoding = /mswin|mingw/ =~ RUBY_PLATFORM ? Encoding::UTF_8 : Encoding.find("locale")
-    assert_equal(encoding, v.encoding)
+    assert_equal(ENCODING, v.encoding)
   end
 
   def test_has_value2
@@ -494,12 +497,20 @@ class TestEnv < Test::Unit::TestCase
     ENV["baz"] = "qux"
     ENV.update({"baz"=>"quux","a"=>"b"})
     check(ENV.to_hash.to_a, [%w(foo bar), %w(baz quux), %w(a b)])
+    ENV.update
+    check(ENV.to_hash.to_a, [%w(foo bar), %w(baz quux), %w(a b)])
+    ENV.update({"foo"=>"zot"}, {"c"=>"d"})
+    check(ENV.to_hash.to_a, [%w(foo zot), %w(baz quux), %w(a b), %w(c d)])
 
     ENV.clear
     ENV["foo"] = "bar"
     ENV["baz"] = "qux"
     ENV.update({"baz"=>"quux","a"=>"b"}) {|k, v1, v2| k + "_" + v1 + "_" + v2 }
     check(ENV.to_hash.to_a, [%w(foo bar), %w(baz baz_qux_quux), %w(a b)])
+    ENV.update {|k, v1, v2| k + "_" + v1 + "_" + v2 }
+    check(ENV.to_hash.to_a, [%w(foo bar), %w(baz baz_qux_quux), %w(a b)])
+    ENV.update({"foo"=>"zot"}, {"c"=>"d"}) {|k, v1, v2| k + "_" + v1 + "_" + v2 }
+    check(ENV.to_hash.to_a, [%w(foo foo_bar_zot), %w(baz baz_qux_quux), %w(a b), %w(c d)])
   end
 
   def test_huge_value
@@ -514,7 +525,7 @@ class TestEnv < Test::Unit::TestCase
     assert_equal(huge_value, ENV["foo"])
   end
 
-  if /mswin|mingw/ =~ RUBY_PLATFORM
+  if windows
     def windows_version
       @windows_version ||= %x[ver][/Version (\d+)/, 1].to_i
     end
@@ -680,37 +691,6 @@ class TestEnv < Test::Unit::TestCase
     end;
   end
 
-  def test_clone_in_ractor
-    assert_ractor(<<-"end;")
-      r = Ractor.new do
-        original_warning_state = Warning[:deprecated]
-        Warning[:deprecated] = false
-
-        begin
-          Ractor.yield ENV.clone.object_id
-          Ractor.yield ENV.clone(freeze: false).object_id
-          Ractor.yield ENV.clone(freeze: nil).object_id
-
-          #{str_for_yielding_exception_class("ENV.clone(freeze: true)")}
-          #{str_for_yielding_exception_class("ENV.clone(freeze: 1)")}
-          #{str_for_yielding_exception_class("ENV.clone(foo: false)")}
-          #{str_for_yielding_exception_class("ENV.clone(1)")}
-          #{str_for_yielding_exception_class("ENV.clone(1, foo: false)")}
-
-        ensure
-          Warning[:deprecated] = original_warning_state
-        end
-      end
-      assert_equal(ENV.object_id, r.take)
-      assert_equal(ENV.object_id, r.take)
-      assert_equal(ENV.object_id, r.take)
-      #{str_for_assert_raise_on_yielded_exception_class(TypeError, "r")}
-      4.times do
-        #{str_for_assert_raise_on_yielded_exception_class(ArgumentError, "r")}
-      end
-    end;
-  end
-
   def test_has_value_in_ractor
     assert_ractor(<<-"end;")
       r = Ractor.new do
