@@ -1268,6 +1268,15 @@ class TestHash < Test::Unit::TestCase
     assert_equal(@cls[a: 10, b: 2, c: 3, d: 4, e: 5, f: 6, g: 7, h: 8, i: 9, j: 10], h)
   end
 
+  def test_update_on_identhash
+    key = +'a'
+    i = @cls[].compare_by_identity
+    i[key] = 0
+    h = @cls[].update(i)
+    key.upcase!
+    assert_equal(0, h.fetch('a'))
+  end
+
   def test_merge
     h1 = @cls[1=>2, 3=>4]
     h2 = {1=>3, 5=>7}
@@ -1290,10 +1299,10 @@ class TestHash < Test::Unit::TestCase
     expected[7] = 8
     h2 = h.merge(7=>8)
     assert_equal(expected, h2)
-    assert_equal(true, h2.compare_by_identity?)
+    assert_predicate(h2, :compare_by_identity?)
     h2 = h.merge({})
     assert_equal(h, h2)
-    assert_equal(true, h2.compare_by_identity?)
+    assert_predicate(h2, :compare_by_identity?)
 
     h = @cls[]
     h.compare_by_identity
@@ -1301,10 +1310,10 @@ class TestHash < Test::Unit::TestCase
     h1.compare_by_identity
     h2 = h.merge(7=>8)
     assert_equal(h1, h2)
-    assert_equal(true, h2.compare_by_identity?)
+    assert_predicate(h2, :compare_by_identity?)
     h2 = h.merge({})
     assert_equal(h, h2)
-    assert_equal(true, h2.compare_by_identity?)
+    assert_predicate(h2, :compare_by_identity?)
   end
 
   def test_merge!
@@ -1458,6 +1467,16 @@ class TestHash < Test::Unit::TestCase
     assert_predicate(h.dup, :compare_by_identity?, bug8703)
   end
 
+  def test_compare_by_identy_memory_leak
+    assert_no_memory_leak([], "", "#{<<~"begin;"}\n#{<<~'end;'}", "[Bug #20145]", rss: true)
+    begin;
+      h = { 1 => 2 }.compare_by_identity
+      1_000_000.times do
+        h.select { false }
+      end
+    end;
+  end
+
   def test_same_key
     bug9646 = '[ruby-dev:48047] [Bug #9646] Infinite loop at Hash#each'
     h = @cls[a=[], 1]
@@ -2333,4 +2352,18 @@ class TestHashOnly < Test::Unit::TestCase
       end
     end;
   end
+
+  def test_ar_to_st_reserved_value
+    klass = Class.new do
+      attr_reader :hash
+      def initialize(val) = @hash = val
+    end
+
+    values = 0.downto(-16).to_a
+    hash = {}
+    values.each do |val|
+      hash[klass.new(val)] = val
+    end
+    assert_equal values, hash.values, "[ruby-core:121239] [Bug #21170]"
+  end
 end
