@@ -668,6 +668,15 @@ class TestMarshal < Test::Unit::TestCase
     end
   end
 
+  def test_recursive_userdef
+    t = Time.utc(0)
+    str = "b".b
+    t.instance_eval {@v = t}
+    assert_raise_with_message(RuntimeError, /recursive\b.*\b_dump/) do
+      Marshal.dump(t)
+    end
+  end
+
   def test_unloadable_usrmarshal
     c = eval("class UsrMarshal\u{23F0 23F3}<Time;self;end")
     c.class_eval {
