@@ -3,7 +3,7 @@
 require_relative "helper"
 require "rubygems"
 
-class TestExit < Gem::TestCase
+class TestGemExit < Gem::TestCase
   def test_exit
     system(*ruby_with_rubygems_in_load_path, "-e", "raise Gem::SystemExitException.new(2)")
     assert_equal 2, $?.exitstatus
