@@ -1,6 +1,7 @@
 # frozen_string_literal: true
-require 'rubygems/test_case'
-require 'rubygems/commands/search_command'
+
+require_relative "helper"
+require "rubygems/commands/search_command"
 
 class TestGemCommandsSearchCommand < Gem::TestCase
   def setup
