@@ -100,7 +100,7 @@ class TestGemRDoc < Gem::TestCase
 
     assert_equal @a.base_dir, e.directory
   ensure
-    FileUtils.chmod(0755, @a.base_dir) if File.directory?(@a.base_dir)
+    FileUtils.chmod(0o755, @a.base_dir) if File.directory?(@a.base_dir)
   end
 
   def test_ri_installed?
@@ -130,8 +130,8 @@ class TestGemRDoc < Gem::TestCase
     assert_equal @a.doc_dir, e.directory
   ensure
     if File.exist? @a.doc_dir
-      FileUtils.chmod 0755, @a.doc_dir
+      FileUtils.chmod 0o755, @a.doc_dir
       FileUtils.rm_r @a.doc_dir
     end
   end
-end
+end if defined?(Gem::RDoc)
