@@ -8,8 +8,8 @@ class TestTempfile < Test::Unit::TestCase
     @tempfile = nil
   end
 
-  def tempfile(*args, &block)
-    t = Tempfile.new(*args, &block)
+  def tempfile(*args, **kw, &block)
+    t = Tempfile.new(*args, **kw, &block)
     @tempfile = (t unless block)
   end
 
@@ -31,15 +31,6 @@ class TestTempfile < Test::Unit::TestCase
     assert_equal "hello world", File.read(path)
   end
 
-  def test_saves_in_dir_tmpdir_by_default
-    t = tempfile("foo")
-    assert_equal Dir.tmpdir, File.dirname(t.path)
-    bug3733 = '[ruby-dev:42089]'
-    assert_nothing_raised(SecurityError, bug3733) {
-      proc {$SAFE = 1; File.expand_path(Dir.tmpdir)}.call
-    }
-  end
-
   def test_saves_in_given_directory
     subdir = File.join(Dir.tmpdir, "tempfile-test-#{rand}")
     Dir.mkdir(subdir)
@@ -72,6 +63,22 @@ class TestTempfile < Test::Unit::TestCase
     assert_match(/\.txt$/, File.basename(t.path))
   end
 
+  def test_dup
+    t = tempfile
+    t2 = t.dup
+    t2.close
+    assert_equal true, t2.closed?
+    assert_equal false, t.closed?
+  end
+
+  def test_clone
+    t = tempfile
+    t2 = t.clone
+    t2.close
+    assert_equal true, t2.closed?
+    assert_equal false, t.closed?
+  end
+
   def test_unlink
     t = tempfile("foo")
     path = t.path
@@ -218,8 +225,7 @@ puts Tempfile.new('foo').path
 
   def test_tempfile_finalizer_does_not_run_if_unlinked
     bug8768 = '[ruby-core:56521] [Bug #8768]'
-    args = %w(--disable-gems -rtempfile)
-    assert_in_out_err(args, <<-'EOS') do |(filename), (error)|
+    assert_in_out_err(%w(-rtempfile), <<-'EOS') do |(filename), (error)|
       tmp = Tempfile.new('foo')
       puts tmp.path
       tmp.close
@@ -359,7 +365,7 @@ puts Tempfile.new('foo').path
     f.close
     assert_file.exist?(path)
   ensure
-    f.close if f && !f.closed?
+    f&.close
     File.unlink path if path
   end
 
@@ -372,30 +378,51 @@ puts Tempfile.new('foo').path
     assert_file.not_exist?(path)
   end
 
-  TRAVERSAL_PATH = Array.new(Dir.pwd.split('/').count, '..').join('/') + Dir.pwd + '/'
+  def test_open
+    Tempfile.open {|f|
+      file = f.open
+      assert_kind_of File, file
+      assert_equal f.to_i, file.to_i
+    }
+  end
 
   def test_open_traversal_dir
-    expect = Dir.glob(TRAVERSAL_PATH + '*').count
-    t = Tempfile.open([TRAVERSAL_PATH, 'foo'])
-    actual = Dir.glob(TRAVERSAL_PATH + '*').count
-    assert_equal expect, actual
-  ensure
-    t.close!
+    assert_mktmpdir_traversal do |traversal_path|
+      t = Tempfile.open([traversal_path, 'foo'])
+      t.path
+    ensure
+      t&.close!
+    end
   end
 
   def test_new_traversal_dir
-    expect = Dir.glob(TRAVERSAL_PATH + '*').count
-    t = Tempfile.new(TRAVERSAL_PATH + 'foo')
-    actual = Dir.glob(TRAVERSAL_PATH + '*').count
-    assert_equal expect, actual
-  ensure
-    t.close!
+    assert_mktmpdir_traversal do |traversal_path|
+      t = Tempfile.new(traversal_path + 'foo')
+      t.path
+    ensure
+      t&.close!
+    end
   end
 
   def test_create_traversal_dir
-    expect = Dir.glob(TRAVERSAL_PATH + '*').count
-    Tempfile.create(TRAVERSAL_PATH + 'foo')
-    actual = Dir.glob(TRAVERSAL_PATH + '*').count
-    assert_equal expect, actual
+    assert_mktmpdir_traversal do |traversal_path|
+      t = Tempfile.create(traversal_path + 'foo')
+      t.path
+    ensure
+      if t
+        t.close
+        File.unlink(t.path)
+      end
+    end
+  end
+
+  def assert_mktmpdir_traversal
+    Dir.mktmpdir do |target|
+      target = target.chomp('/') + '/'
+      traversal_path = target.sub(/\A\w:/, '') # for DOSISH
+      traversal_path = Array.new(target.count('/')-2, '..').join('/') + traversal_path
+      actual = yield traversal_path
+      assert_not_send([File.absolute_path(actual), :start_with?, target])
+    end
   end
 end
