@@ -6,9 +6,9 @@ class Colorize
   #   Colorize.new(color: color, colors_file: colors_file)
   def initialize(color = nil, opts = ((_, color = color, nil)[0] if Hash === color))
     @colors = @reset = nil
-    @color = (opts[:color] if opts)
-    if color or (color == nil && STDOUT.tty?)
-      if (/\A\e\[.*m\z/ =~ IO.popen("tput smso", "r", :err => IO::NULL, &:read) rescue nil)
+    @color = opts && opts[:color] || color
+    if color or (color == nil && coloring?)
+      if (%w[smso so].any? {|attr| /\A\e\[.*m\z/ =~ IO.popen("tput #{attr}", "r", :err => IO::NULL, &:read)} rescue nil)
         @beg = "\e["
         colors = (colors = ENV['TEST_COLORS']) ? Hash[colors.scan(/(\w+)=([^:\n]*)/)] : {}
         if opts and colors_file = opts[:colors_file]
@@ -27,21 +27,47 @@ class Colorize
   end
 
   DEFAULTS = {
-    "pass"=>"32", "fail"=>"31;1", "skip"=>"33;1",
+    # color names
     "black"=>"30", "red"=>"31", "green"=>"32", "yellow"=>"33",
     "blue"=>"34", "magenta"=>"35", "cyan"=>"36", "white"=>"37",
     "bold"=>"1", "underline"=>"4", "reverse"=>"7",
+    "bright_black"=>"90", "bright_red"=>"91", "bright_green"=>"92", "bright_yellow"=>"93",
+    "bright_blue"=>"94", "bright_magenta"=>"95", "bright_cyan"=>"96", "bright_white"=>"97",
+
+    # abstract decorations
+    "pass"=>"green", "fail"=>"red;bold", "skip"=>"yellow;bold", "note"=>"bright_yellow",
   }
 
+  def coloring?
+    STDOUT.tty? && (!(nc = ENV['NO_COLOR']) || nc.empty?)
+  end
+
   # colorize.decorate(str, name = color_name)
   def decorate(str, name = @color)
-    if @colors and color = (@colors[name] || DEFAULTS[name])
+    if coloring? and color = resolve_color(name)
       "#{@beg}#{color}m#{str}#{@reset}"
     else
       str
     end
   end
 
+  def resolve_color(color = @color, seen = {}, colors = nil)
+    return unless @colors
+    color.to_s.gsub(/\b[a-z][\w ]+/) do |n|
+      n.gsub!(/\W+/, "_")
+      n.downcase!
+      c = seen[n] and next c
+      if colors
+        c = colors[n]
+      elsif (c = (tbl = @colors)[n] || (tbl = DEFAULTS)[n])
+        colors = tbl
+      else
+        next n
+      end
+      seen[n] = resolve_color(c, seen, colors)
+    end
+  end
+
   DEFAULTS.each_key do |name|
     define_method(name) {|str|
       decorate(str, name)
