@@ -11,6 +11,7 @@
 
 #include "eval_intern.h"
 #include "internal.h"
+#include "internal/class.h"
 #include "internal/error.h"
 #include "internal/vm.h"
 #include "iseq.h"
@@ -56,8 +57,11 @@ calc_pos(const rb_iseq_t *iseq, const VALUE *pc, int *lineno, int *node_id)
         VM_ASSERT(ISEQ_BODY(iseq)->iseq_size);
 
         ptrdiff_t n = pc - ISEQ_BODY(iseq)->iseq_encoded;
-        VM_ASSERT(n <= ISEQ_BODY(iseq)->iseq_size);
         VM_ASSERT(n >= 0);
+#if SIZEOF_PTRDIFF_T > SIZEOF_INT
+        VM_ASSERT(n <= (ptrdiff_t)UINT_MAX);
+#endif
+        VM_ASSERT((unsigned int)n <= ISEQ_BODY(iseq)->iseq_size);
         ASSUME(n >= 0);
         size_t pos = n; /* no overflow */
         if (LIKELY(pos)) {
@@ -67,7 +71,7 @@ calc_pos(const rb_iseq_t *iseq, const VALUE *pc, int *lineno, int *node_id)
 #if VMDEBUG && defined(HAVE_BUILTIN___BUILTIN_TRAP)
         else {
             /* SDR() is not possible; that causes infinite loop. */
-            rb_print_backtrace();
+            rb_print_backtrace(stderr);
             __builtin_trap();
         }
 #endif
@@ -155,17 +159,14 @@ location_mark_entry(rb_backtrace_location_t *fi)
     }
 }
 
-static size_t
-location_memsize(const void *ptr)
-{
-    /* rb_backtrace_location_t *fi = (rb_backtrace_location_t *)ptr; */
-    return sizeof(rb_backtrace_location_t);
-}
-
 static const rb_data_type_t location_data_type = {
     "frame_info",
-    {location_mark, RUBY_TYPED_DEFAULT_FREE, location_memsize,},
-    0, 0, RUBY_TYPED_FREE_IMMEDIATELY
+    {
+        location_mark,
+        RUBY_TYPED_DEFAULT_FREE,
+        NULL, // No external memory to report,
+    },
+    0, 0, RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED | RUBY_TYPED_EMBEDDABLE
 };
 
 int
@@ -178,7 +179,7 @@ static inline rb_backtrace_location_t *
 location_ptr(VALUE locobj)
 {
     struct valued_frame_info *vloc;
-    GetCoreDataFromValue(locobj, struct valued_frame_info, vloc);
+    TypedData_Get_Struct(locobj, struct valued_frame_info, &location_data_type, vloc);
     return vloc->loc;
 }
 
@@ -451,10 +452,10 @@ location_inspect_m(VALUE self)
 }
 
 typedef struct rb_backtrace_struct {
-    rb_backtrace_location_t *backtrace;
     int backtrace_size;
     VALUE strary;
     VALUE locary;
+    rb_backtrace_location_t backtrace[1];
 } rb_backtrace_t;
 
 static void
@@ -470,14 +471,6 @@ backtrace_mark(void *ptr)
     rb_gc_mark_movable(bt->locary);
 }
 
-static void
-backtrace_free(void *ptr)
-{
-   rb_backtrace_t *bt = (rb_backtrace_t *)ptr;
-   if (bt->backtrace) ruby_xfree(bt->backtrace);
-   ruby_xfree(bt);
-}
-
 static void
 location_update_entry(rb_backtrace_location_t *fi)
 {
@@ -508,17 +501,15 @@ backtrace_update(void *ptr)
     bt->locary = rb_gc_location(bt->locary);
 }
 
-static size_t
-backtrace_memsize(const void *ptr)
-{
-    rb_backtrace_t *bt = (rb_backtrace_t *)ptr;
-    return sizeof(rb_backtrace_t) + sizeof(rb_backtrace_location_t) * bt->backtrace_size;
-}
-
 static const rb_data_type_t backtrace_data_type = {
     "backtrace",
-    {backtrace_mark, backtrace_free, backtrace_memsize, backtrace_update},
-    0, 0, RUBY_TYPED_FREE_IMMEDIATELY
+    {
+        backtrace_mark,
+        RUBY_DEFAULT_FREE,
+        NULL, // No external memory to report,
+        backtrace_update,
+    },
+    0, 0, RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED | RUBY_TYPED_EMBEDDABLE
 };
 
 int
@@ -590,11 +581,10 @@ rb_ec_partial_backtrace_object(const rb_execution_context_t *ec, long start_fram
     const rb_control_frame_t *cfp = ec->cfp;
     const rb_control_frame_t *end_cfp = RUBY_VM_END_CONTROL_FRAME(ec);
     ptrdiff_t size;
-    rb_backtrace_t *bt;
-    VALUE btobj = backtrace_alloc(rb_cBacktrace);
+    rb_backtrace_t *bt = NULL;
+    VALUE btobj = Qnil;
     rb_backtrace_location_t *loc = NULL;
     unsigned long cfunc_counter = 0;
-    GetCoreDataFromValue(btobj, rb_backtrace_t, bt);
 
     // In the case the thread vm_stack or cfp is not initialized, there is no backtrace.
     if (end_cfp == NULL) {
@@ -622,7 +612,10 @@ rb_ec_partial_backtrace_object(const rb_execution_context_t *ec, long start_fram
         }
     }
 
-    bt->backtrace = ZALLOC_N(rb_backtrace_location_t, num_frames);
+    size_t memsize = offsetof(rb_backtrace_t, backtrace) + num_frames * sizeof(rb_backtrace_location_t);
+    btobj = rb_data_typed_object_zalloc(rb_cBacktrace, memsize, &backtrace_data_type);
+    TypedData_Get_Struct(btobj, rb_backtrace_t, &backtrace_data_type, bt);
+
     bt->backtrace_size = 0;
     if (num_frames == 0) {
         if (start_too_large) *start_too_large = 0;
@@ -640,7 +633,7 @@ rb_ec_partial_backtrace_object(const rb_execution_context_t *ec, long start_fram
                     const VALUE *pc = cfp->pc;
                     loc = &bt->backtrace[bt->backtrace_size++];
                     loc->type = LOCATION_TYPE_ISEQ;
-                    loc->iseq = iseq;
+                    RB_OBJ_WRITE(btobj, &loc->iseq, iseq);
                     loc->pc = pc;
                     bt_update_cfunc_loc(cfunc_counter, loc-1, iseq, pc);
                     if (do_yield) {
@@ -670,6 +663,7 @@ rb_ec_partial_backtrace_object(const rb_execution_context_t *ec, long start_fram
         for (; cfp != end_cfp; cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp)) {
             if (cfp->iseq && cfp->pc && (!skip_internal || !is_internal_location(cfp))) {
                 bt_update_cfunc_loc(cfunc_counter, loc, cfp->iseq, cfp->pc);
+                RB_OBJ_WRITTEN(btobj, Qundef, cfp->iseq);
                 if (do_yield) {
                     bt_yield_loc(loc - cfunc_counter, cfunc_counter, btobj);
                 }
@@ -682,7 +676,7 @@ rb_ec_partial_backtrace_object(const rb_execution_context_t *ec, long start_fram
     return btobj;
 }
 
-MJIT_FUNC_EXPORTED VALUE
+VALUE
 rb_ec_backtrace_object(const rb_execution_context_t *ec)
 {
     return rb_ec_partial_backtrace_object(ec, BACKTRACE_START, ALL_BACKTRACE_LINES, NULL, FALSE, FALSE);
@@ -715,7 +709,7 @@ backtrace_to_str_ary(VALUE self)
 {
     VALUE r;
     rb_backtrace_t *bt;
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
     r = backtrace_collect(bt, location_to_str_dmyarg, 0);
     RB_GC_GUARD(self);
     return r;
@@ -725,21 +719,21 @@ VALUE
 rb_backtrace_to_str_ary(VALUE self)
 {
     rb_backtrace_t *bt;
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
 
     if (!bt->strary) {
-        bt->strary = backtrace_to_str_ary(self);
+        RB_OBJ_WRITE(self, &bt->strary, backtrace_to_str_ary(self));
     }
     return bt->strary;
 }
 
-MJIT_FUNC_EXPORTED void
+void
 rb_backtrace_use_iseq_first_lineno_for_last_location(VALUE self)
 {
-    const rb_backtrace_t *bt;
+    rb_backtrace_t *bt;
     rb_backtrace_location_t *loc;
 
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
     VM_ASSERT(bt->backtrace_size > 0);
 
     loc = &bt->backtrace[0];
@@ -757,7 +751,7 @@ location_create(rb_backtrace_location_t *srcloc, void *btobj)
     obj = TypedData_Make_Struct(rb_cBacktraceLocation, struct valued_frame_info, &location_data_type, vloc);
 
     vloc->loc = srcloc;
-    vloc->btobj = (VALUE)btobj;
+    RB_OBJ_WRITE(obj, &vloc->btobj, (VALUE)btobj);
 
     return obj;
 }
@@ -767,7 +761,7 @@ backtrace_to_location_ary(VALUE self)
 {
     VALUE r;
     rb_backtrace_t *bt;
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
     r = backtrace_collect(bt, location_create, (void *)self);
     RB_GC_GUARD(self);
     return r;
@@ -777,10 +771,10 @@ VALUE
 rb_backtrace_to_location_ary(VALUE self)
 {
     rb_backtrace_t *bt;
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
 
     if (!bt->locary) {
-        bt->locary = backtrace_to_location_ary(self);
+        RB_OBJ_WRITE(self, &bt->locary, backtrace_to_location_ary(self));
     }
     return bt->locary;
 }
@@ -796,8 +790,8 @@ static VALUE
 backtrace_load_data(VALUE self, VALUE str)
 {
     rb_backtrace_t *bt;
-    GetCoreDataFromValue(self, rb_backtrace_t, bt);
-    bt->strary = str;
+    TypedData_Get_Struct(self, rb_backtrace_t, &backtrace_data_type, bt);
+    RB_OBJ_WRITE(self, &bt->strary, str);
     return self;
 }
 
@@ -998,31 +992,38 @@ vm_backtrace_print(FILE *fp)
                    &arg);
 }
 
+struct oldbt_bugreport_arg {
+    FILE *fp;
+    int count;
+};
+
 static void
 oldbt_bugreport(void *arg, VALUE file, int line, VALUE method)
 {
+    struct oldbt_bugreport_arg *p = arg;
+    FILE *fp = p->fp;
     const char *filename = NIL_P(file) ? "ruby" : RSTRING_PTR(file);
-    if (!*(int *)arg) {
-        fprintf(stderr, "-- Ruby level backtrace information "
+    if (!p->count) {
+        fprintf(fp, "-- Ruby level backtrace information "
                 "----------------------------------------\n");
-        *(int *)arg = 1;
+        p->count = 1;
     }
     if (NIL_P(method)) {
-        fprintf(stderr, "%s:%d:in unknown method\n", filename, line);
+        fprintf(fp, "%s:%d:in unknown method\n", filename, line);
     }
     else {
-        fprintf(stderr, "%s:%d:in `%s'\n", filename, line, RSTRING_PTR(method));
+        fprintf(fp, "%s:%d:in `%s'\n", filename, line, RSTRING_PTR(method));
     }
 }
 
 void
-rb_backtrace_print_as_bugreport(void)
+rb_backtrace_print_as_bugreport(FILE *fp)
 {
     struct oldbt_arg arg;
-    int i = 0;
+    struct oldbt_bugreport_arg barg = {fp, 0};
 
     arg.func = oldbt_bugreport;
-    arg.data = (int *)&i;
+    arg.data = &barg;
 
     backtrace_each(GET_EC(),
                    oldbt_init,
@@ -1572,12 +1573,12 @@ rb_debug_inspector_backtrace_locations(const rb_debug_inspector_t *dc)
     return dc->backtrace;
 }
 
-int
-rb_profile_frames(int start, int limit, VALUE *buff, int *lines)
+static int
+thread_profile_frames(rb_execution_context_t *ec, int start, int limit, VALUE *buff, int *lines)
 {
     int i;
-    const rb_execution_context_t *ec = GET_EC();
     const rb_control_frame_t *cfp = ec->cfp, *end_cfp = RUBY_VM_END_CONTROL_FRAME(ec);
+    const rb_control_frame_t *top = cfp;
     const rb_callable_method_entry_t *cme;
 
     // If this function is called inside a thread after thread creation, but
@@ -1608,7 +1609,18 @@ rb_profile_frames(int start, int limit, VALUE *buff, int *lines)
                 buff[i] = (VALUE)cfp->iseq;
             }
 
-            if (lines) lines[i] = calc_lineno(cfp->iseq, cfp->pc);
+            if (lines) {
+                // The topmost frame may not have an updated PC because the JIT
+                // may not have set one.  The JIT compiler will update the PC
+                // before entering a new function (so that `caller` will work),
+                // so only the topmost frame could possibly have an out of date PC
+                if (cfp == top && cfp->jit_return) {
+                    lines[i] = 0;
+                }
+                else {
+                    lines[i] = calc_lineno(cfp->iseq, cfp->pc);
+                }
+            }
 
             i++;
         }
@@ -1626,6 +1638,27 @@ rb_profile_frames(int start, int limit, VALUE *buff, int *lines)
     return i;
 }
 
+int
+rb_profile_frames(int start, int limit, VALUE *buff, int *lines)
+{
+    rb_execution_context_t *ec = rb_current_execution_context(false);
+
+    // If there is no EC, we may be attempting to profile a non-Ruby thread or a
+    // M:N shared native thread which has no active Ruby thread.
+    if (!ec) {
+        return 0;
+    }
+
+    return thread_profile_frames(ec, start, limit, buff, lines);
+}
+
+int
+rb_profile_thread_frames(VALUE thread, int start, int limit, VALUE *buff, int *lines)
+{
+    rb_thread_t *th = rb_thread_ptr(thread);
+    return thread_profile_frames(th->ec, start, limit, buff, lines);
+}
+
 static const rb_iseq_t *
 frame2iseq(VALUE frame)
 {
@@ -1745,7 +1778,7 @@ rb_profile_frame_classpath(VALUE frame)
             klass = RBASIC(klass)->klass;
         }
         else if (FL_TEST(klass, FL_SINGLETON)) {
-            klass = rb_ivar_get(klass, id__attached__);
+            klass = RCLASS_ATTACHED_OBJECT(klass);
             if (!RB_TYPE_P(klass, T_CLASS) && !RB_TYPE_P(klass, T_MODULE))
                 return rb_sprintf("#<%s:%p>", rb_class2name(rb_obj_class(klass)), (void*)klass);
         }
