@@ -45,7 +45,6 @@
 #define OPT_CHECKED_RUN              1
 #define OPT_INLINE_METHOD_CACHE      1
 #define OPT_GLOBAL_METHOD_CACHE      1
-#define OPT_BLOCKINLINING            0
 
 #ifndef OPT_IC_FOR_IVAR
 #define OPT_IC_FOR_IVAR 1
@@ -55,17 +54,12 @@
 #define OPT_OPERANDS_UNIFICATION     1
 #define OPT_INSTRUCTIONS_UNIFICATION 0
 #define OPT_UNIFY_ALL_COMBINATION    0
-#define OPT_STACK_CACHING            0
 
 /* misc */
 #ifndef OPT_SUPPORT_JOKE
 #define OPT_SUPPORT_JOKE             0
 #endif
 
-#ifndef OPT_SUPPORT_CALL_C_FUNCTION
-#define OPT_SUPPORT_CALL_C_FUNCTION  0
-#endif
-
 #ifndef VM_COLLECT_USAGE_DETAILS
 #define VM_COLLECT_USAGE_DETAILS     0
 #endif
